@@ -67,8 +67,16 @@ func pathCredsCreate(b *databaseBackend) []*framework.Path {
 }
 
 func (b *databaseBackend) pathCredsCreateRead() framework.OperationFunc {
-	return func(ctx context.Context, req *logical.Request, data *framework.FieldData) (*logical.Response, error) {
+	return func(ctx context.Context, req *logical.Request, data *framework.FieldData) (resp *logical.Response, err error) {
 		name := data.Get("name").(string)
+		modified := false
+		defer func() {
+			if err == nil && (resp == nil || !resp.IsError()) {
+				b.dbEvent(ctx, "creds-create", req.Path, name, modified)
+			} else {
+				b.dbEvent(ctx, "creds-create-fail", req.Path, name, modified)
+			}
+		}()
 
 		// Get the role
 		role, err := b.Role(ctx, req.Storage, name)
@@ -202,6 +210,7 @@ func (b *databaseBackend) pathCredsCreateRead() framework.OperationFunc {
 			b.CloseIfShutdown(dbi, err)
 			return nil, err
 		}
+		modified = true
 		respData["username"] = newUserResp.Username
 
 		// Database plugins using the v4 interface generate and return the password.
@@ -216,7 +225,7 @@ func (b *databaseBackend) pathCredsCreateRead() framework.OperationFunc {
 			"db_name":               role.DBName,
 			"revocation_statements": role.Statements.Revocation,
 		}
-		resp := b.Secret(SecretCredsType).Response(respData, internal)
+		resp = b.Secret(SecretCredsType).Response(respData, internal)
 		resp.Secret.TTL = role.DefaultTTL
 		resp.Secret.MaxTTL = role.MaxTTL
 		return resp, nil
@@ -259,6 +268,10 @@ func (b *databaseBackend) pathStaticCredsRead() framework.OperationFunc {
 			if role.StaticAccount.RotationWindow.Seconds() != 0 {
 				respData["rotation_window"] = role.StaticAccount.RotationWindow.Seconds()
 			}
+
+			// The schedule is in UTC, but we want to convert it to the local time
+			role.StaticAccount.Schedule.Location = time.Local
+			respData["ttl"] = role.StaticAccount.CredentialTTL().Seconds()
 		}
 
 		switch role.CredentialType {
