@@ -12,6 +12,7 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/hashicorp/go-secure-stdlib/parseutil"
 	"github.com/hashicorp/vault/sdk/framework"
 	"github.com/hashicorp/vault/sdk/helper/errutil"
 	"github.com/hashicorp/vault/sdk/helper/keysutil"
@@ -51,7 +52,7 @@ type batchResponseSignItem struct {
 
 // BatchRequestVerifyItem represents a request item for batch processing.
 // A map type allows us to distinguish between empty and missing values.
-type batchRequestVerifyItem map[string]string
+type batchRequestVerifyItem map[string]interface{}
 
 // BatchResponseVerifyItem represents a response item for batch processing
 type batchResponseVerifyItem struct {
@@ -216,6 +217,11 @@ derivation is enabled; currently only available with ed25519 keys.`,
 				Description: "The HMAC, including vault header/key version",
 			},
 
+			"cmac": {
+				Type:        framework.TypeString,
+				Description: "The CMAC, including vault header/key version",
+			},
+
 			"input": {
 				Type:        framework.TypeString,
 				Description: "The base64-encoded input data to verify",
@@ -226,6 +232,11 @@ derivation is enabled; currently only available with ed25519 keys.`,
 				Description: `Hash algorithm to use (POST URL parameter)`,
 			},
 
+			"mac_length": {
+				Type:        framework.TypeInt,
+				Description: `MAC length to use (POST body parameter). Valid values are:`,
+			},
+
 			"hash_algorithm": {
 				Type:    framework.TypeString,
 				Default: defaultHashAlgorithm,
@@ -279,7 +290,7 @@ Options are 'auto' (the default used by Golang, causing the salt to be as large
 			"batch_input": {
 				Type: framework.TypeSlice,
 				Description: `Specifies a list of items for processing. When this parameter is set,
-any supplied  'input', 'hmac' or 'signature' parameters will be ignored. Responses are returned in the
+any supplied  'input', 'hmac', 'cmac' or 'signature' parameters will be ignored. Responses are returned in the
 'batch_results' array component of the 'data' element of the response. Any batch output will
 preserve the order of the batch input`,
 			},
@@ -534,6 +545,9 @@ func (b *backend) pathVerifyWrite(ctx context.Context, req *logical.Request, d *
 		if hmac, ok := d.GetOk("hmac"); ok {
 			batchInputItems[0]["hmac"] = hmac.(string)
 		}
+		if cmac, ok := d.GetOk("cmac"); ok {
+			batchInputItems[0]["cmac"] = cmac.(string)
+		}
 		batchInputItems[0]["context"] = d.Get("context").(string)
 	}
 
@@ -542,26 +556,30 @@ func (b *backend) pathVerifyWrite(ctx context.Context, req *logical.Request, d *
 	// If one batch_input item is 'hmac', they all must be 'hmac'.
 	sigFound := false
 	hmacFound := false
+	cmacFound := false
 	missing := false
 	for _, v := range batchInputItems {
 		if _, ok := v["signature"]; ok {
 			sigFound = true
 		} else if _, ok := v["hmac"]; ok {
 			hmacFound = true
+		} else if _, ok := v["cmac"]; ok {
+			cmacFound = true
 		} else {
 			missing = true
 		}
 	}
+	optionsSet := numBooleansTrue(sigFound, hmacFound, cmacFound)
 
 	switch {
-	case batchInputRaw == nil && sigFound && hmacFound:
-		return logical.ErrorResponse("provide one of 'signature' or 'hmac'"), logical.ErrInvalidRequest
+	case batchInputRaw == nil && optionsSet > 1:
+		return logical.ErrorResponse("provide one of 'signature', 'hmac' or 'cmac'"), logical.ErrInvalidRequest
 
-	case batchInputRaw == nil && !sigFound && !hmacFound:
-		return logical.ErrorResponse("neither a 'signature' nor an 'hmac' were given to verify"), logical.ErrInvalidRequest
+	case batchInputRaw == nil && optionsSet == 0:
+		return logical.ErrorResponse("missing 'signature', 'hmac' or 'cmac' were given to verify"), logical.ErrInvalidRequest
 
-	case sigFound && hmacFound:
-		return logical.ErrorResponse("elements of batch_input must all provide 'signature' or all provide 'hmac'"), logical.ErrInvalidRequest
+	case optionsSet > 1:
+		return logical.ErrorResponse("elements of batch_input must all provide either 'signature', 'hmac' or 'cmac'"), logical.ErrInvalidRequest
 
 	case missing && sigFound:
 		return logical.ErrorResponse("some elements of batch_input are missing 'signature'"), logical.ErrInvalidRequest
@@ -569,11 +587,17 @@ func (b *backend) pathVerifyWrite(ctx context.Context, req *logical.Request, d *
 	case missing && hmacFound:
 		return logical.ErrorResponse("some elements of batch_input are missing 'hmac'"), logical.ErrInvalidRequest
 
-	case missing:
-		return logical.ErrorResponse("no batch_input elements have 'signature' or 'hmac'"), logical.ErrInvalidRequest
+	case missing && cmacFound:
+		return logical.ErrorResponse("some elements of batch_input are missing 'cmac'"), logical.ErrInvalidRequest
+
+	case optionsSet == 0:
+		return logical.ErrorResponse("no batch_input elements have 'signature', 'hmac' or 'cmac'"), logical.ErrInvalidRequest
 
 	case hmacFound:
 		return b.pathHMACVerify(ctx, req, d)
+
+	case cmacFound:
+		return b.pathCMACVerify(ctx, req, d)
 	}
 
 	name := d.Get("name").(string)
@@ -636,27 +660,38 @@ func (b *backend) pathVerifyWrite(ctx context.Context, req *logical.Request, d *
 	response := make([]batchResponseVerifyItem, len(batchInputItems))
 
 	for i, item := range batchInputItems {
-
 		rawInput, ok := item["input"]
 		if !ok {
 			response[i].Error = "missing input"
 			response[i].err = logical.ErrInvalidRequest
 			continue
 		}
+		strInput, err := parseutil.ParseString(rawInput)
+		if err != nil {
+			response[i].Error = fmt.Sprintf("unable to parse input as string: %s", err)
+			response[i].err = logical.ErrInvalidRequest
+			continue
+		}
 
-		input, err := base64.StdEncoding.DecodeString(rawInput)
+		input, err := base64.StdEncoding.DecodeString(strInput)
 		if err != nil {
 			response[i].Error = fmt.Sprintf("unable to decode input as base64: %s", err)
 			response[i].err = logical.ErrInvalidRequest
 			continue
 		}
 
-		sig, ok := item["signature"]
+		sigRaw, ok := item["signature"].(string)
 		if !ok {
 			response[i].Error = "missing signature"
 			response[i].err = logical.ErrInvalidRequest
 			continue
 		}
+		sig, err := parseutil.ParseString(sigRaw)
+		if err != nil {
+			response[i].Error = fmt.Sprintf("failed to parse signature as a string: %s", err)
+			response[i].err = logical.ErrInvalidRequest
+			continue
+		}
 
 		if p.Type.HashSignatureInput() && !prehashed {
 			hf := keysutil.HashFuncMap[hashAlgorithm]()
@@ -666,7 +701,12 @@ func (b *backend) pathVerifyWrite(ctx context.Context, req *logical.Request, d *
 			}
 		}
 
-		contextRaw := item["context"]
+		contextRaw, err := parseutil.ParseString(item["context"])
+		if err != nil {
+			response[i].Error = fmt.Sprintf("failed to parse context as a string: %s", err)
+			response[i].err = logical.ErrInvalidRequest
+			continue
+		}
 		var context []byte
 		if len(contextRaw) != 0 {
 			context, err = base64.StdEncoding.DecodeString(contextRaw)
@@ -720,7 +760,9 @@ func (b *backend) pathVerifyWrite(ctx context.Context, req *logical.Request, d *
 	if batchInputRaw != nil {
 		// Copy the references
 		for i := range batchInputItems {
-			response[i].Reference = batchInputItems[i]["reference"]
+			if ref, err := parseutil.ParseString(batchInputItems[i]["reference"]); err == nil {
+				response[i].Reference = ref
+			}
 		}
 		resp.Data = map[string]interface{}{
 			"batch_results": response,
@@ -740,6 +782,53 @@ func (b *backend) pathVerifyWrite(ctx context.Context, req *logical.Request, d *
 	return resp, nil
 }
 
+func numBooleansTrue(bools ...bool) int {
+	numSet := 0
+	for _, value := range bools {
+		if value {
+			numSet++
+		}
+	}
+	return numSet
+}
+
+func decodeTransitSignature(sig string) ([]byte, int, error) {
+	if !strings.HasPrefix(sig, "vault:v") {
+		return nil, 0, fmt.Errorf("prefix is not vault:v")
+	}
+
+	splitVerification := strings.SplitN(strings.TrimPrefix(sig, "vault:v"), ":", 2)
+	if len(splitVerification) != 2 {
+		return nil, 0, fmt.Errorf("wrong number of fields delimited by ':', got %d expected 2", len(splitVerification))
+	}
+
+	ver, err := strconv.Atoi(splitVerification[0])
+	if err != nil {
+		return nil, 0, fmt.Errorf("key version number %s count not be decoded", splitVerification[0])
+	}
+
+	if ver < 1 {
+		return nil, 0, fmt.Errorf("key version less than 1 are invalid got: %d", ver)
+	}
+
+	if len(strings.TrimSpace(splitVerification[1])) == 0 {
+		return nil, 0, fmt.Errorf("missing base64 verification string from vault signature")
+	}
+
+	verBytes, err := base64.StdEncoding.DecodeString(splitVerification[1])
+	if err != nil {
+		return nil, 0, fmt.Errorf("unable to decode verification string as base64: %s", err)
+	}
+
+	return verBytes, ver, nil
+}
+
+func encodeTransitSignature(value []byte, keyVersion int) string {
+	retStr := base64.StdEncoding.EncodeToString(value)
+	retStr = fmt.Sprintf("vault:v%d:%s", keyVersion, retStr)
+	return retStr
+}
+
 const pathSignHelpSyn = `Generate a signature for input data using the named key`
 
 const pathSignHelpDesc = `
