@@ -11,6 +11,7 @@ import (
 	"net/http/httptest"
 	"os"
 	"strings"
+	sync "sync/atomic"
 	"testing"
 	"time"
 
@@ -387,8 +388,9 @@ func TestServerRun(t *testing.T) {
 			}
 
 			errCh := make(chan error)
+			serverErrCh := make(chan error, 1)
 			go func() {
-				errCh <- server.Run(ctx, templateTokenCh, templatesToRender)
+				errCh <- server.Run(ctx, templateTokenCh, templatesToRender, &sync.Bool{}, serverErrCh)
 			}()
 
 			// send a dummy value to trigger the internal Runner to query for secret
@@ -492,8 +494,9 @@ func TestNewServerLogLevels(t *testing.T) {
 			defer cancel()
 
 			errCh := make(chan error)
+			serverErrCh := make(chan error, 1)
 			go func() {
-				errCh <- server.Run(ctx, templateTokenCh, templatesToRender)
+				errCh <- server.Run(ctx, templateTokenCh, templatesToRender, &sync.Bool{}, serverErrCh)
 			}()
 
 			// send a dummy value to trigger auth so the server will exit
