@@ -8,8 +8,8 @@ import (
 	"fmt"
 	"strings"
 
+	"github.com/hashicorp/cli"
 	"github.com/hashicorp/go-secure-stdlib/strutil"
-	"github.com/mitchellh/cli"
 	"github.com/posener/complete"
 )
 
