@@ -10,11 +10,11 @@ import (
 	"os"
 	"strings"
 
+	"github.com/hashicorp/cli"
 	"github.com/hashicorp/go-secure-stdlib/password"
 	"github.com/hashicorp/vault/api"
 	"github.com/hashicorp/vault/helper/pgpkeys"
 	"github.com/hashicorp/vault/sdk/helper/roottoken"
-	"github.com/mitchellh/cli"
 	"github.com/posener/complete"
 )
 
@@ -23,6 +23,7 @@ var (
 	_ cli.CommandAutocomplete = (*OperatorGenerateRootCommand)(nil)
 )
 
+//go:generate enumer -type=generateRootKind -trimprefix=generateRoot
 type generateRootKind int
 
 const (
