@@ -11,8 +11,8 @@ import (
 	"strings"
 	"time"
 
+	"github.com/hashicorp/cli"
 	"github.com/hashicorp/vault/api"
-	"github.com/mitchellh/cli"
 	"github.com/posener/complete"
 	"github.com/ryanuber/columnize"
 )
@@ -56,7 +56,7 @@ func (c *OperatorUsageCommand) Flags() *FlagSets {
 
 	f.TimeVar(&TimeVar{
 		Name:       "start-time",
-		Usage:      "Start of report period. Defaults to 'default_reporting_period' before end time.",
+		Usage:      "Start of report period. Defaults to billing start time",
 		Target:     &c.flagStartTime,
 		Completion: complete.PredictNothing,
 		Default:    time.Time{},
@@ -64,7 +64,7 @@ func (c *OperatorUsageCommand) Flags() *FlagSets {
 	})
 	f.TimeVar(&TimeVar{
 		Name:       "end-time",
-		Usage:      "End of report period. Defaults to end of last month.",
+		Usage:      "End of report period. Defaults to end of the current month.",
 		Target:     &c.flagEndTime,
 		Completion: complete.PredictNothing,
 		Default:    time.Time{},
@@ -132,7 +132,7 @@ func (c *OperatorUsageCommand) Run(args []string) int {
 	c.outputTimestamps(resp.Data)
 
 	out := []string{
-		"Namespace path | Distinct entities | Non-Entity tokens | Active clients",
+		"Namespace path | Entity Clients | Non-Entity clients | Secret syncs | ACME clients | Active clients",
 	}
 
 	out = append(out, c.namespacesOutput(resp.Data)...)
@@ -142,6 +142,12 @@ func (c *OperatorUsageCommand) Run(args []string) int {
 	colConfig.Empty = " " // Do not show n/a on intentional blank lines
 	colConfig.Glue = "   "
 	c.UI.Output(tableOutput(out, colConfig))
+
+	// Also, output the warnings returned, if any:
+	for _, warning := range resp.Warnings {
+		c.UI.Warn(warning)
+	}
+
 	return 0
 }
 
@@ -196,8 +202,9 @@ type UsageResponse struct {
 	entityCount   int64
 	// As per 1.9, the tokenCount field will contain the distinct non-entity
 	// token clients instead of each individual token.
-	tokenCount int64
-
+	tokenCount  int64
+	secretSyncs int64
+	acmeCount   int64
 	clientCount int64
 }
 
@@ -232,16 +239,22 @@ func (c *OperatorUsageCommand) parseNamespaceCount(rawVal interface{}) (UsageRes
 		return ret, errors.New("missing counts")
 	}
 
-	ret.entityCount, ok = jsonNumberOK(counts, "distinct_entities")
+	ret.entityCount, ok = jsonNumberOK(counts, "entity_clients")
 	if !ok {
-		return ret, errors.New("missing distinct_entities")
+		return ret, errors.New("missing entity_clients")
 	}
 
-	ret.tokenCount, ok = jsonNumberOK(counts, "non_entity_tokens")
+	ret.tokenCount, ok = jsonNumberOK(counts, "non_entity_clients")
 	if !ok {
-		return ret, errors.New("missing non_entity_tokens")
+		return ret, errors.New("missing non_entity_clients")
 	}
 
+	// don't error if the secret syncs key is missing
+	ret.secretSyncs, _ = jsonNumberOK(counts, "secret_syncs")
+
+	// don't error if acme clients is missing
+	ret.acmeCount, _ = jsonNumberOK(counts, "acme_clients")
+
 	ret.clientCount, ok = jsonNumberOK(counts, "clients")
 	if !ok {
 		return ret, errors.New("missing clients")
@@ -274,8 +287,8 @@ func (c *OperatorUsageCommand) namespacesOutput(data map[string]interface{}) []s
 			sortOrder = "2" + val.namespacePath
 		}
 
-		formattedLine := fmt.Sprintf("%s | %d | %d | %d",
-			val.namespacePath, val.entityCount, val.tokenCount, val.clientCount)
+		formattedLine := fmt.Sprintf("%s | %d | %d | %d | %d | %d",
+			val.namespacePath, val.entityCount, val.tokenCount, val.secretSyncs, val.acmeCount, val.clientCount)
 		nsOut = append(nsOut, UsageCommandNamespace{
 			formattedLine: formattedLine,
 			sortOrder:     sortOrder,
@@ -296,7 +309,7 @@ func (c *OperatorUsageCommand) namespacesOutput(data map[string]interface{}) []s
 
 func (c *OperatorUsageCommand) totalOutput(data map[string]interface{}) []string {
 	// blank line separating it from namespaces
-	out := []string{"  |  |  |  "}
+	out := []string{"  |  |  |  |  |  "}
 
 	total, ok := data["total"].(map[string]interface{})
 	if !ok {
@@ -304,24 +317,30 @@ func (c *OperatorUsageCommand) totalOutput(data map[string]interface{}) []string
 		return out
 	}
 
-	entityCount, ok := jsonNumberOK(total, "distinct_entities")
+	entityCount, ok := jsonNumberOK(total, "entity_clients")
 	if !ok {
-		c.UI.Error("missing distinct_entities in total")
+		c.UI.Error("missing entity_clients in total")
 		return out
 	}
 
-	tokenCount, ok := jsonNumberOK(total, "non_entity_tokens")
+	tokenCount, ok := jsonNumberOK(total, "non_entity_clients")
 	if !ok {
-		c.UI.Error("missing non_entity_tokens in total")
+		c.UI.Error("missing non_entity_clients in total")
 		return out
 	}
+	// don't error if secret syncs key is missing
+	secretSyncs, _ := jsonNumberOK(total, "secret_syncs")
+
+	// don't error if acme clients is missing
+	acmeCount, _ := jsonNumberOK(total, "acme_clients")
+
 	clientCount, ok := jsonNumberOK(total, "clients")
 	if !ok {
 		c.UI.Error("missing clients in total")
 		return out
 	}
 
-	out = append(out, fmt.Sprintf("Total | %d | %d | %d",
-		entityCount, tokenCount, clientCount))
+	out = append(out, fmt.Sprintf("Total | %d | %d | %d | %d | %d",
+		entityCount, tokenCount, secretSyncs, acmeCount, clientCount))
 	return out
 }
