@@ -6,8 +6,8 @@ package command
 import (
 	"strings"
 
+	"github.com/hashicorp/cli"
 	"github.com/hashicorp/vault/version"
-	"github.com/mitchellh/cli"
 	"github.com/posener/complete"
 )
 
