@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 # Copyright (c) HashiCorp, Inc.
 # SPDX-License-Identifier: BUSL-1.1
 
@@ -11,11 +11,14 @@ export CGO_ENABLED=0
 
 root_dir="$(git rev-parse --show-toplevel)"
 pushd "$root_dir" > /dev/null
-make ci-build-ui ci-build
 
-: "${BIN_PATH:="dist"}"
-: "${BUNDLE_PATH:=$(git rev-parse --show-toplevel)/vault.zip}"
-echo "--> Bundling $BIN_PATH/* to $BUNDLE_PATH"
-zip -r -j "$BUNDLE_PATH" "$BIN_PATH/"
+if [ -n "$BUILD_UI" ] && [ "$BUILD_UI" = "true" ]; then
+  make ci-build-ui
+fi
+
+make ci-build
 
 popd > /dev/null
+
+echo "--> Bundling $BIN_PATH/* to $BUNDLE_PATH"
+zip -r -j "$BUNDLE_PATH" "$BIN_PATH/"
