@@ -2,16 +2,18 @@
 # Copyright (c) HashiCorp, Inc.
 # SPDX-License-Identifier: BUSL-1.1
 
-
 set -e
 
-binpath=${VAULT_INSTALL_DIR}/vault
-
 fail() {
   echo "$1" 1>&2
   return 1
 }
 
+[[ -z "$VAULT_PROXY_ADDRESS" ]] && fail "VAULT_ADDR env variable has not been set"
+[[ -z "$VAULT_PROXY_PIDFILE" ]] && fail "VAULT_ADDR env variable has not been set"
+[[ -z "$VAULT_INSTALL_DIR" ]] && fail "VAULT_INSTALL_DIR env variable has not been set"
+
+binpath=${VAULT_INSTALL_DIR}/vault
 test -x "$binpath" || fail "unable to locate vault binary at $binpath"
 
 # Will cause the Vault CLI to communicate with the Vault Proxy, since it
@@ -26,7 +28,9 @@ unset VAULT_TOKEN
 # var) to lookup the details of the Proxy's token and make sure that the
 # .data.path field contains 'auth/approle/login', thus confirming that the Proxy
 # automatically authenticated itself.
-$binpath token lookup -format=json | jq -r '.data.path' | grep -q 'auth/approle/login'
+if ! $binpath token lookup -format=json | jq -Mer --arg expected "auth/approle/login" '.data.path == $expected'; then
+  fail "expected proxy to automatically authenticate using 'auth/approle/login', got: '$($binpath token lookup -format=json | jq -r '.data.path')'"
+fi
 
 # Now that we're done, kill the proxy
 pkill -F "${VAULT_PROXY_PIDFILE}" || true
