@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 # Copyright (c) HashiCorp, Inc.
 # SPDX-License-Identifier: BUSL-1.1
 
@@ -7,6 +7,9 @@ function fail() {
   exit 1
 }
 
+# Exit early if we haven't been given an expected DEFAULT_LCQ
+[[ -z "$DEFAULT_LCQ" ]] && exit 0
+
 [[ -z "$RETRY_INTERVAL" ]] && fail "RETRY_INTERVAL env variable has not been set"
 [[ -z "$TIMEOUT_SECONDS" ]] && fail "TIMEOUT_SECONDS env variable has not been set"
 [[ -z "$VAULT_ADDR" ]] && fail "VAULT_ADDR env variable has not been set"
@@ -43,4 +46,4 @@ while [ "$(date +%s)" -lt "$end_time" ]; do
   sleep "$RETRY_INTERVAL"
 done
 
-fail "Timed out waiting for Default LCQ verification to complete. Data:\n\t$(getMaxLeases)"
\ No newline at end of file
+fail "Timed out waiting for Default LCQ verification to complete. Data:\n\t$(getMaxLeases)"
