@@ -7,6 +7,7 @@ package osutil
 
 import (
 	"io/fs"
+	"os"
 )
 
 func FileUidMatch(info fs.FileInfo, path string, uid int) error {
@@ -17,3 +18,7 @@ func FileUidMatch(info fs.FileInfo, path string, uid int) error {
 func Umask(newmask int) int {
 	return 0
 }
+
+func Chown(f *os.File, owner, group int) error {
+	return nil
+}
