@@ -76,8 +76,8 @@ func IsCurrentMonth(t, compare time.Time) bool {
 	return queryMonthStart.Equal(thisMonthStart)
 }
 
-// GetMostRecentContinuousMonths finds the start time of the most
-// recent set of continguous months.
+// GetMostRecentContiguousMonths finds the start time of the most
+// recent set of continuous months.
 //
 // For example, if the most recent start time is Aug 15, then that range is just 1 month
 // If the recent start times are Aug 1 and July 1 and June 15, then that range is
@@ -179,3 +179,13 @@ func (_ DefaultClock) NewTicker(d time.Duration) *time.Ticker {
 func (_ DefaultClock) NewTimer(d time.Duration) *time.Timer {
 	return time.NewTimer(d)
 }
+
+// NormalizeToYear returns date normalized to the latest date
+// within one year of normal. Assumes the date argument is
+// some date before normal.
+func NormalizeToYear(date, normal time.Time) time.Time {
+	for date.AddDate(1, 0, 0).Compare(normal) <= 0 {
+		date = date.AddDate(1, 0, 0)
+	}
+	return date
+}
