@@ -18,24 +18,23 @@ import (
 	"testing"
 	"time"
 
+	"github.com/go-test/deep"
+	log "github.com/hashicorp/go-hclog"
 	kv "github.com/hashicorp/vault-plugin-secrets-kv"
 	"github.com/hashicorp/vault/api"
-	auditFile "github.com/hashicorp/vault/builtin/audit/file"
+	"github.com/hashicorp/vault/audit"
 	credUserpass "github.com/hashicorp/vault/builtin/credential/userpass"
+	"github.com/hashicorp/vault/helper/namespace"
 	"github.com/hashicorp/vault/helper/testhelpers/corehelpers"
 	"github.com/hashicorp/vault/internalshared/configutil"
+	"github.com/hashicorp/vault/sdk/framework"
 	"github.com/hashicorp/vault/sdk/helper/consts"
 	"github.com/hashicorp/vault/sdk/helper/logging"
 	"github.com/hashicorp/vault/sdk/logical"
 	"github.com/hashicorp/vault/sdk/physical"
 	"github.com/hashicorp/vault/sdk/physical/inmem"
-
-	"github.com/go-test/deep"
-	log "github.com/hashicorp/go-hclog"
-
-	"github.com/hashicorp/vault/audit"
-	"github.com/hashicorp/vault/helper/namespace"
 	"github.com/hashicorp/vault/vault"
+	"github.com/stretchr/testify/require"
 )
 
 func TestLogical(t *testing.T) {
@@ -64,9 +63,10 @@ func TestLogical(t *testing.T) {
 		"data": map[string]interface{}{
 			"data": "bar",
 		},
-		"auth":      nil,
-		"wrap_info": nil,
-		"warnings":  nilWarnings,
+		"auth":       nil,
+		"wrap_info":  nil,
+		"warnings":   nilWarnings,
+		"mount_type": "kv",
 	}
 	testResponseStatus(t, resp, 200)
 	testResponseBody(t, resp, &actual)
@@ -181,9 +181,10 @@ func TestLogical_StandbyRedirect(t *testing.T) {
 			"entity_id":        "",
 			"type":             "service",
 		},
-		"warnings":  nilWarnings,
-		"wrap_info": nil,
-		"auth":      nil,
+		"warnings":   nilWarnings,
+		"wrap_info":  nil,
+		"auth":       nil,
+		"mount_type": "token",
 	}
 
 	testResponseStatus(t, resp, 200)
@@ -222,6 +223,7 @@ func TestLogical_CreateToken(t *testing.T) {
 		"renewable":      false,
 		"lease_duration": json.Number("0"),
 		"data":           nil,
+		"mount_type":     "token",
 		"wrap_info":      nil,
 		"auth": map[string]interface{}{
 			"policies":        []interface{}{"root"},
@@ -350,7 +352,7 @@ func TestLogical_ListSuffix(t *testing.T) {
 	req = req.WithContext(namespace.RootContext(nil))
 	req.Header.Add(consts.AuthHeaderName, rootToken)
 
-	_, _, status, err = buildLogicalRequestNoAuth(core.PerfStandby(), nil, req)
+	_, _, status, err = buildLogicalRequestNoAuth(core.PerfStandby(), core.RouterAccess(), nil, req)
 	if err != nil || status != 0 {
 		t.Fatal(err)
 	}
@@ -367,6 +369,94 @@ func TestLogical_ListSuffix(t *testing.T) {
 	}
 }
 
+// TestLogical_BinaryPath tests the legacy behavior passing in binary data to a
+// path that isn't explicitly marked by a plugin as a binary path to fail, along
+// with making sure we pass through when marked as a binary path
+func TestLogical_BinaryPath(t *testing.T) {
+	t.Parallel()
+
+	testHandler := func(ctx context.Context, l *logical.Request, data *framework.FieldData) (*logical.Response, error) {
+		return nil, nil
+	}
+	operations := map[logical.Operation]framework.OperationHandler{
+		logical.PatchOperation:  &framework.PathOperation{Callback: testHandler},
+		logical.UpdateOperation: &framework.PathOperation{Callback: testHandler},
+	}
+
+	conf := &vault.CoreConfig{
+		BuiltinRegistry: corehelpers.NewMockBuiltinRegistry(),
+		LogicalBackends: map[string]logical.Factory{
+			"bintest": func(ctx context.Context, config *logical.BackendConfig) (logical.Backend, error) {
+				b := new(framework.Backend)
+				b.BackendType = logical.TypeLogical
+				b.Paths = []*framework.Path{
+					{Pattern: "binary", Operations: operations},
+					{Pattern: "binary/" + framework.MatchAllRegex("test"), Operations: operations},
+				}
+				b.PathsSpecial = &logical.Paths{Binary: []string{"binary", "binary/*"}}
+				err := b.Setup(ctx, config)
+				return b, err
+			},
+		},
+	}
+
+	core, _, token := vault.TestCoreUnsealedWithConfig(t, conf)
+	ln, addr := TestServer(t, core)
+	defer ln.Close()
+	TestServerAuth(t, addr, token)
+
+	mountReq := &logical.Request{
+		Operation:   logical.UpdateOperation,
+		ClientToken: token,
+		Path:        "sys/mounts/bintest",
+		Data: map[string]interface{}{
+			"type": "bintest",
+		},
+	}
+	mountResp, err := core.HandleRequest(namespace.RootContext(nil), mountReq)
+	if err != nil {
+		t.Fatalf("failed mounting bin-test engine: %v", err)
+	}
+	if mountResp.IsError() {
+		t.Fatalf("failed mounting bin-test error in response: %v", mountResp.Error())
+	}
+
+	tests := []struct {
+		name           string
+		op             string
+		url            string
+		expectedReturn int
+	}{
+		{name: "PUT non-binary", op: "PUT", url: addr + "/v1/bintest/non-binary", expectedReturn: http.StatusBadRequest},
+		{name: "POST non-binary", op: "POST", url: addr + "/v1/bintest/non-binary", expectedReturn: http.StatusBadRequest},
+		{name: "PUT binary", op: "PUT", url: addr + "/v1/bintest/binary", expectedReturn: http.StatusNoContent},
+		{name: "POST binary", op: "POST", url: addr + "/v1/bintest/binary/sub-path", expectedReturn: http.StatusNoContent},
+	}
+	for _, test := range tests {
+		t.Run(test.name, func(st *testing.T) {
+			var resp *http.Response
+			switch test.op {
+			case "PUT":
+				resp = testHttpPutBinaryData(st, token, test.url, make([]byte, 100))
+			case "POST":
+				resp = testHttpPostBinaryData(st, token, test.url, make([]byte, 100))
+			default:
+				t.Fatalf("unsupported operation: %s", test.op)
+			}
+			testResponseStatus(st, resp, test.expectedReturn)
+			if test.expectedReturn != http.StatusNoContent {
+				all, err := io.ReadAll(resp.Body)
+				if err != nil {
+					st.Fatalf("failed reading error response body: %v", err)
+				}
+				if !strings.Contains(string(all), "error parsing JSON") {
+					st.Fatalf("error response body did not contain expected error: %v", all)
+				}
+			}
+		})
+	}
+}
+
 func TestLogical_ListWithQueryParameters(t *testing.T) {
 	core, _, rootToken := vault.TestCoreUnsealed(t)
 
@@ -465,12 +555,12 @@ func TestLogical_RespondWithStatusCode(t *testing.T) {
 		t.Fatalf("Bad Status code: %d", w.Code)
 	}
 
-	bodyRaw, err := ioutil.ReadAll(w.Body)
+	bodyRaw, err := io.ReadAll(w.Body)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	expected := `{"request_id":"id","lease_id":"","renewable":false,"lease_duration":0,"data":{"test-data":"foo"},"wrap_info":null,"warnings":null,"auth":null}`
+	expected := `{"request_id":"id","lease_id":"","renewable":false,"lease_duration":0,"data":{"test-data":"foo"},"wrap_info":null,"warnings":null,"auth":null,"mount_type":""}`
 
 	if string(bodyRaw[:]) != strings.Trim(expected, "\n") {
 		t.Fatalf("bad response: %s", string(bodyRaw[:]))
@@ -478,13 +568,11 @@ func TestLogical_RespondWithStatusCode(t *testing.T) {
 }
 
 func TestLogical_Audit_invalidWrappingToken(t *testing.T) {
-	t.Setenv("VAULT_AUDIT_DISABLE_EVENTLOGGER", "true")
-
 	// Create a noop audit backend
-	noop := corehelpers.TestNoopAudit(t, nil)
+	noop := audit.TestNoopAudit(t, "noop/", nil)
 	c, _, root := vault.TestCoreUnsealedWithConfig(t, &vault.CoreConfig{
 		AuditBackends: map[string]audit.Factory{
-			"noop": func(ctx context.Context, config *audit.BackendConfig, _ bool, _ audit.HeaderFormatter) (audit.Backend, error) {
+			"noop": func(config *audit.BackendConfig, _ audit.HeaderFormatter) (audit.Backend, error) {
 				return noop, nil
 			},
 		},
@@ -493,7 +581,6 @@ func TestLogical_Audit_invalidWrappingToken(t *testing.T) {
 	defer ln.Close()
 
 	// Enable the audit backend
-
 	resp := testHttpPost(t, root, addr+"/v1/sys/audit/noop", map[string]interface{}{
 		"type": "noop",
 	})
@@ -594,7 +681,7 @@ func TestLogical_AuditPort(t *testing.T) {
 			"kv": kv.VersionedKVFactory,
 		},
 		AuditBackends: map[string]audit.Factory{
-			"file": auditFile.Factory,
+			"file": audit.NewFileBackend,
 		},
 	}
 
@@ -658,9 +745,12 @@ func TestLogical_AuditPort(t *testing.T) {
 
 	decoder := json.NewDecoder(auditLogFile)
 
-	var auditRecord map[string]interface{}
 	count := 0
-	for decoder.Decode(&auditRecord) == nil {
+	for decoder.More() {
+		var auditRecord map[string]interface{}
+		err := decoder.Decode(&auditRecord)
+		require.NoError(t, err)
+
 		count += 1
 
 		// Skip the first line
@@ -765,14 +855,25 @@ func TestLogical_ErrRelativePath(t *testing.T) {
 }
 
 func testBuiltinPluginMetadataAuditLog(t *testing.T, log map[string]interface{}, expectedMountClass string) {
+	t.Helper()
+
 	if mountClass, ok := log["mount_class"].(string); !ok {
 		t.Fatalf("mount_class should be a string, not %T", log["mount_class"])
 	} else if mountClass != expectedMountClass {
 		t.Fatalf("bad: mount_class should be %s, not %s", expectedMountClass, mountClass)
 	}
 
-	if _, ok := log["mount_running_version"].(string); !ok {
-		t.Fatalf("mount_running_version should be a string, not %T", log["mount_running_version"])
+	// Requests have 'mount_running_version' but Responses have 'mount_running_plugin_version'
+	runningVersionRaw, runningVersionRawOK := log["mount_running_version"]
+	runningPluginVersionRaw, runningPluginVersionRawOK := log["mount_running_plugin_version"]
+	if !runningVersionRawOK && !runningPluginVersionRawOK {
+		t.Fatalf("mount_running_version/mount_running_plugin_version should be present")
+	} else if runningVersionRawOK {
+		if _, ok := runningVersionRaw.(string); !ok {
+			t.Fatalf("mount_running_version should be string, not %T", runningVersionRaw)
+		}
+	} else if _, ok := runningPluginVersionRaw.(string); !ok {
+		t.Fatalf("mount_running_plugin_version should be string, not %T", runningPluginVersionRaw)
 	}
 
 	if _, ok := log["mount_running_sha256"].(string); ok {
@@ -789,7 +890,7 @@ func testBuiltinPluginMetadataAuditLog(t *testing.T, log map[string]interface{},
 func TestLogical_AuditEnabled_ShouldLogPluginMetadata_Auth(t *testing.T) {
 	coreConfig := &vault.CoreConfig{
 		AuditBackends: map[string]audit.Factory{
-			"file": auditFile.Factory,
+			"file": audit.NewFileBackend,
 		},
 	}
 
@@ -819,38 +920,45 @@ func TestLogical_AuditEnabled_ShouldLogPluginMetadata_Auth(t *testing.T) {
 			"file_path": auditLogFile.Name(),
 		},
 	})
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 
 	_, err = c.Logical().Write("auth/token/create", map[string]interface{}{
 		"ttl": "10s",
 	})
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
+
+	// Disable audit now we're done performing operations
+	err = c.Sys().DisableAudit("file")
+	require.NoError(t, err)
 
 	// Check the audit trail on request and response
 	decoder := json.NewDecoder(auditLogFile)
-	var auditRecord map[string]interface{}
-	for decoder.Decode(&auditRecord) == nil {
-		auditRequest := map[string]interface{}{}
-		if req, ok := auditRecord["request"]; ok {
-			auditRequest = req.(map[string]interface{})
-			if auditRequest["path"] != "auth/token/create" {
-				continue
-			}
-		}
-		testBuiltinPluginMetadataAuditLog(t, auditRequest, consts.PluginTypeCredential.String())
+	for decoder.More() {
+		var auditRecord map[string]interface{}
+		err := decoder.Decode(&auditRecord)
+		require.NoError(t, err)
 
-		auditResponse := map[string]interface{}{}
-		if req, ok := auditRecord["response"]; ok {
-			auditRequest = req.(map[string]interface{})
-			if auditResponse["path"] != "auth/token/create" {
+		if req, ok := auditRecord["request"]; ok {
+			auditRequest, ok := req.(map[string]interface{})
+			require.True(t, ok)
+
+			path, ok := auditRequest["path"].(string)
+			require.True(t, ok)
+
+			if path != "auth/token/create" {
 				continue
 			}
+
+			testBuiltinPluginMetadataAuditLog(t, auditRequest, consts.PluginTypeCredential.String())
+		}
+
+		// Should never have a response without a corresponding request.
+		if resp, ok := auditRecord["response"]; ok {
+			auditResponse, ok := resp.(map[string]interface{})
+			require.True(t, ok)
+
+			testBuiltinPluginMetadataAuditLog(t, auditResponse, consts.PluginTypeCredential.String())
 		}
-		testBuiltinPluginMetadataAuditLog(t, auditResponse, consts.PluginTypeCredential.String())
 	}
 }
 
@@ -862,7 +970,7 @@ func TestLogical_AuditEnabled_ShouldLogPluginMetadata_Secret(t *testing.T) {
 			"kv": kv.VersionedKVFactory,
 		},
 		AuditBackends: map[string]audit.Factory{
-			"file": auditFile.Factory,
+			"file": audit.NewFileBackend,
 		},
 	}
 
@@ -888,9 +996,7 @@ func TestLogical_AuditEnabled_ShouldLogPluginMetadata_Secret(t *testing.T) {
 	// Enable the audit backend
 	tempDir := t.TempDir()
 	auditLogFile, err := os.CreateTemp(tempDir, "")
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 
 	err = c.Sys().EnableAuditWithOptions("file", &api.EnableAuditOptions{
 		Type: "file",
@@ -898,9 +1004,7 @@ func TestLogical_AuditEnabled_ShouldLogPluginMetadata_Secret(t *testing.T) {
 			"file_path": auditLogFile.Name(),
 		},
 	})
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 
 	{
 		writeData := map[string]interface{}{
@@ -917,26 +1021,36 @@ func TestLogical_AuditEnabled_ShouldLogPluginMetadata_Secret(t *testing.T) {
 		})
 	}
 
+	// Disable audit now we're done performing operations
+	err = c.Sys().DisableAudit("file")
+	require.NoError(t, err)
+
 	// Check the audit trail on request and response
 	decoder := json.NewDecoder(auditLogFile)
-	var auditRecord map[string]interface{}
-	for decoder.Decode(&auditRecord) == nil {
-		auditRequest := map[string]interface{}{}
-		if req, ok := auditRecord["request"]; ok {
-			auditRequest = req.(map[string]interface{})
-			if auditRequest["path"] != "kv/data/foo" {
-				continue
-			}
-		}
-		testBuiltinPluginMetadataAuditLog(t, auditRequest, consts.PluginTypeSecrets.String())
+	for decoder.More() {
+		var auditRecord map[string]interface{}
+		err := decoder.Decode(&auditRecord)
+		require.NoError(t, err)
 
-		auditResponse := map[string]interface{}{}
-		if req, ok := auditRecord["response"]; ok {
-			auditRequest = req.(map[string]interface{})
-			if auditResponse["path"] != "kv/data/foo" {
+		if req, ok := auditRecord["request"]; ok {
+			auditRequest, ok := req.(map[string]interface{})
+			require.True(t, ok)
+
+			path, ok := auditRequest["path"]
+			require.True(t, ok)
+
+			if path != "kv/data/foo" {
 				continue
 			}
+
+			testBuiltinPluginMetadataAuditLog(t, auditRequest, consts.PluginTypeSecrets.String())
+		}
+
+		if resp, ok := auditRecord["response"]; ok {
+			auditResponse, ok := resp.(map[string]interface{})
+			require.True(t, ok)
+
+			testBuiltinPluginMetadataAuditLog(t, auditResponse, consts.PluginTypeSecrets.String())
 		}
-		testBuiltinPluginMetadataAuditLog(t, auditResponse, consts.PluginTypeSecrets.String())
 	}
 }
