@@ -1,13 +1,17 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package cassandra
 
 import (
+	"os"
+	"reflect"
+	"testing"
+
 	log "github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/vault/helper/testhelpers/cassandra"
 	"github.com/hashicorp/vault/sdk/helper/logging"
 	"github.com/hashicorp/vault/sdk/physical"
-	"os"
-	"reflect"
-	"testing"
 )
 
 func TestCassandraBackend(t *testing.T) {
@@ -24,10 +28,12 @@ func TestCassandraBackend(t *testing.T) {
 	// Run vault tests
 	logger := logging.NewVaultLogger(log.Debug)
 	b, err := NewCassandraBackend(map[string]string{
-		"hosts":            host.ConnectionURL(),
-		"protocol_version": "3",
+		"hosts":                       host.ConnectionURL(),
+		"protocol_version":            "3",
+		"connection_timeout":          "5",
+		"initial_connection_timeout":  "5",
+		"simple_retry_policy_retries": "3",
 	}, logger)
-
 	if err != nil {
 		t.Fatalf("Failed to create new backend: %v", err)
 	}
@@ -41,7 +47,8 @@ func TestCassandraBackendBuckets(t *testing.T) {
 		"":          {"."},
 		"a":         {"."},
 		"a/b":       {".", "a"},
-		"a/b/c/d/e": {".", "a", "a/b", "a/b/c", "a/b/c/d"}}
+		"a/b/c/d/e": {".", "a", "a/b", "a/b/c", "a/b/c/d"},
+	}
 
 	b := &CassandraBackend{}
 	for input, expected := range expectations {
