@@ -14,13 +14,11 @@ import (
 	"unicode"
 
 	metrics "github.com/armon/go-metrics"
-	"github.com/cockroachdb/cockroach-go/crdb"
+	"github.com/cockroachdb/cockroach-go/v2/crdb"
 	log "github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-multierror"
 	"github.com/hashicorp/go-secure-stdlib/strutil"
 	"github.com/hashicorp/vault/sdk/physical"
-
-	// CockroachDB uses the Postgres SQL driver
 	_ "github.com/jackc/pgx/v4/stdlib"
 )
 
