@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package main
 
 import (
@@ -5,7 +8,7 @@ import (
 	"os"
 
 	"github.com/hashicorp/vault/plugins/database/cassandra"
-	dbplugin "github.com/hashicorp/vault/sdk/database/dbplugin/v5"
+	"github.com/hashicorp/vault/sdk/database/dbplugin/v5"
 )
 
 func main() {
@@ -18,12 +21,7 @@ func main() {
 
 // Run instantiates a Cassandra object, and runs the RPC server for the plugin
 func Run() error {
-	dbType, err := cassandra.New()
-	if err != nil {
-		return err
-	}
-
-	dbplugin.Serve(dbType.(dbplugin.Database))
+	dbplugin.ServeMultiplex(cassandra.New)
 
 	return nil
 }
