@@ -15,6 +15,11 @@ module.exports = {
     },
     'require-input-label': 'off',
     'no-array-prototype-extensions': 'off',
+    // from bump to ember-template-lint@6.0.0
+    'no-builtin-form-components': 'off',
+    'no-at-ember-render-modifiers': 'off',
+    'no-unnecessary-curly-strings': 'off',
+    'no-unnecessary-curly-parens': 'off',
   },
   overrides: [
     {
