@@ -1,12 +1,17 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import IdentityAdapter from './base';
 
 export default IdentityAdapter.extend({
   lookup(store, data) {
-    let url = `/${this.urlPrefix()}/identity/lookup/group`;
-    return this.ajax(url, 'POST', { data }).then(response => {
+    const url = `/${this.urlPrefix()}/identity/lookup/group`;
+    return this.ajax(url, 'POST', { data }).then((response) => {
       // unsuccessful lookup is a 204
       if (!response) return;
-      let modelName = 'identity/group';
+      const modelName = 'identity/group';
       store.push(
         store
           .serializerFor(modelName)
