@@ -3,7 +3,6 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { assign } from '@ember/polyfills';
 import ApplicationAdapter from './application';
 
 export default ApplicationAdapter.extend({
@@ -21,7 +20,7 @@ export default ApplicationAdapter.extend({
     return this.ajax(this.buildURL(type.modelName, name), 'PUT', { data }).then(() => {
       // doing this to make it like a Vault response - ember data doesn't like 204s if it's not a DELETE
       return {
-        data: assign({}, this.serialize(snapshot), { id: name }),
+        data: { ...this.serialize(snapshot), id: name },
       };
     });
   },
