@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import ApplicationAdapter from './application';
 
 export default ApplicationAdapter.extend({
@@ -6,8 +11,8 @@ export default ApplicationAdapter.extend({
   },
 
   fetchStatus(mode) {
-    let url = this.getStatusUrl(mode);
-    return this.ajax(url, 'GET', { unauthenticated: true }).then(resp => {
+    const url = this.getStatusUrl(mode);
+    return this.ajax(url, 'GET', { unauthenticated: true }).then((resp) => {
       return resp.data;
     });
   },
