@@ -4,10 +4,9 @@
  */
 
 import Component from '@glimmer/component';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { allMethods, methods } from 'vault/helpers/mountable-auth-methods';
 import { allEngines, mountableEngines } from 'vault/helpers/mountable-secret-engines';
-import { tracked } from '@glimmer/tracking';
 
 /**
  *
@@ -19,13 +18,12 @@ import { tracked } from '@glimmer/tracking';
  * ```js
  * <MountBackend::TypeForm @setMountType={{this.setMountType}} @mountType="secret" />
  * ```
- * @param {CallableFunction} setMountType - function will recieve the mount type string. Should update the model type value
+ * @param {CallableFunction} setMountType - function will receive the mount type string. Should update the model type value
  * @param {string} [mountType=auth] - mount type can be `auth` or `secret`
  */
 
 export default class MountBackendTypeForm extends Component {
   @service version;
-  @tracked selection;
 
   get secretEngines() {
     return this.version.isEnterprise ? allEngines() : mountableEngines();
