@@ -4,7 +4,7 @@
  */
 
 import TransformBase, { addToList, removeFromList } from './transform-edit-base';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default TransformBase.extend({
   flashMessages: service(),
