@@ -7,8 +7,5 @@ import Component from '@glimmer/component';
 import { tracked } from '@glimmer/tracking';
 
 export default class ExportComponent extends Component {
-  @tracked
-  wrapTTL = null;
-  @tracked
-  exportVersion = false;
+  @tracked exportVersion = false;
 }
