@@ -4,7 +4,7 @@
  */
 
 /* eslint-disable ember/no-observers */
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { alias } from '@ember/object/computed';
 import Controller from '@ember/controller';
 import { observer } from '@ember/object';
@@ -16,6 +16,7 @@ export default Controller.extend({
   permissions: service(),
   namespaceService: service('namespace'),
   flashMessages: service(),
+  customMessages: service(),
 
   vaultVersion: service('version'),
   console: service(),
