@@ -7,6 +7,15 @@ import Controller from '@ember/controller';
 import ListController from 'core/mixins/list-controller';
 
 export default Controller.extend(ListController, {
+  // callback from HDS pagination to set the queryParams page
+  get paginationQueryParams() {
+    return (page) => {
+      return {
+        page,
+      };
+    };
+  },
+
   actions: {
     onDelete() {
       this.send('reload');
