@@ -4,13 +4,15 @@
  */
 
 import Controller from '@ember/controller';
+import { service } from '@ember/service';
 
 export default Controller.extend({
+  router: service(),
   queryParams: ['action'],
   action: '',
   actions: {
     onPromote() {
-      this.transitionToRoute('vault.cluster.replication.mode.index', 'dr');
+      this.router.transitionTo('vault.cluster.replication.mode.index', 'dr');
     },
   },
 });
