@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import Controller, { inject as controller } from '@ember/controller';
 import BackendCrumbMixin from 'vault/mixins/backend-crumb';
 
@@ -9,12 +14,12 @@ export default Controller.extend(BackendCrumbMixin, {
   itemType: '',
 
   actions: {
-    refresh: function() {
+    refresh: function () {
       this.send('refreshModel');
     },
     toggleAdvancedEdit(bool) {
       this.set('preferAdvancedEdit', bool);
-      this.get('backendController').set('preferAdvancedEdit', bool);
+      this.backendController.set('preferAdvancedEdit', bool);
     },
   },
 });
