@@ -1,15 +1,23 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import Controller, { inject as controller } from '@ember/controller';
 import BackendCrumbMixin from 'vault/mixins/backend-crumb';
 
 export default Controller.extend(BackendCrumbMixin, {
   backendController: controller('vault.cluster.secrets.backend'),
-  queryParams: ['version'],
+  queryParams: ['version', 'itemType'],
   version: '',
+  itemType: '',
+
   reset() {
     this.set('version', '');
+    this.set('itemType', '');
   },
   actions: {
-    refresh: function() {
+    refresh: function () {
       // closure actions don't bubble to routes,
       // so we have to manually bubble here
       this.send('refreshModel');
@@ -17,7 +25,7 @@ export default Controller.extend(BackendCrumbMixin, {
 
     toggleAdvancedEdit(bool) {
       this.set('preferAdvancedEdit', bool);
-      this.get('backendController').set('preferAdvancedEdit', bool);
+      this.backendController.set('preferAdvancedEdit', bool);
     },
   },
 });
