@@ -5,19 +5,27 @@
 
 import { or } from '@ember/object/computed';
 import { computed } from '@ember/object';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Controller from '@ember/controller';
 import BackendCrumbMixin from 'vault/mixins/backend-crumb';
-import WithNavToNearestAncestor from 'vault/mixins/with-nav-to-nearest-ancestor';
 import ListController from 'core/mixins/list-controller';
 import { keyIsFolder } from 'core/utils/key-utils';
 
-export default Controller.extend(ListController, BackendCrumbMixin, WithNavToNearestAncestor, {
+export default Controller.extend(ListController, BackendCrumbMixin, {
   flashMessages: service(),
   queryParams: ['page', 'pageFilter', 'tab'],
 
   tab: '',
 
+  // callback from HDS pagination to set the queryParams page
+  get paginationQueryParams() {
+    return (page) => {
+      return {
+        page,
+      };
+    };
+  },
+
   filterIsFolder: computed('filter', function () {
     return !!keyIsFolder(this.filter);
   }),
@@ -43,16 +51,13 @@ export default Controller.extend(ListController, BackendCrumbMixin, WithNavToNea
         });
     },
 
-    delete(item, type) {
+    delete(item) {
       const name = item.id;
       item
         .destroyRecord()
         .then(() => {
           this.flashMessages.success(`${name} was successfully deleted.`);
           this.send('reload');
-          if (type === 'secret') {
-            this.navToNearestAncestor.perform(name);
-          }
         })
         .catch((e) => {
           const error = e.errors ? e.errors.join('. ') : e.message;
