@@ -3,7 +3,7 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Controller from '@ember/controller';
 import { supportedSecretBackends } from 'vault/helpers/supported-secret-backends';
 import { allEngines } from 'vault/helpers/mountable-secret-engines';
@@ -18,7 +18,7 @@ export default class MountSecretBackendController extends Controller {
   onMountSuccess(type, path, useEngineRoute = false) {
     let transition;
     if (SUPPORTED_BACKENDS.includes(type)) {
-      const engineInfo = allEngines().findBy('type', type);
+      const engineInfo = allEngines().find((engine) => engine.type === type);
       if (useEngineRoute) {
         transition = this.router.transitionTo(
           `vault.cluster.secrets.backend.${engineInfo.engineRoute}`,
