@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import { helper as buildHelper } from '@ember/component/helper';
 
 export function filterWildcard([string, array]) {
@@ -7,9 +12,9 @@ export function filterWildcard([string, array]) {
   if (!string.id && string) {
     string = { id: string };
   }
-  let stringId = string.id;
-  const filterBy = stringId =>
-    array.filter(item => new RegExp('^' + stringId.replace(/\*/g, '.*') + '$').test(item));
+  const stringId = string.id;
+  const filterBy = (stringId) =>
+    array.filter((item) => new RegExp('^' + stringId.replace(/\*/g, '.*') + '$').test(item));
   return filterBy(stringId).length;
 }
 
