@@ -5,7 +5,7 @@
 
 /* eslint-disable ember/no-observers */
 import Helper from '@ember/component/helper';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { observer } from '@ember/object';
 
 export default Helper.extend({
