@@ -1,6 +1,11 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import { helper as buildHelper } from '@ember/component/helper';
 
-export function isWildcardString(string) {
+export function isWildcardString([string]) {
   if (!string) {
     return false;
   }
