@@ -10,7 +10,7 @@ export function initialize() {
   registerDeprecationHandler((message, options, next) => {
     // filter deprecations that are scheduled to be removed in a specific version
     // when upgrading or addressing deprecation warnings be sure to update this or remove if not needed
-    if (options?.until.includes('5.0')) {
+    if (options?.until.includes('5.0') && options?.id.startsWith('ember-data')) {
       return;
     }
     next(message, options);
