@@ -10,8 +10,8 @@ import lazyCapabilities, { apiPath } from 'vault/macros/lazy-capabilities';
 export default Model.extend({
   approved: attr('boolean'),
   requestPath: attr('string'),
-  requestEntity: belongsTo('identity/entity', { async: false }),
-  authorizations: hasMany('identity/entity', { async: false }),
+  requestEntity: belongsTo('identity/entity', { async: false, inverse: null }),
+  authorizations: hasMany('identity/entity', { async: false, inverse: null }),
 
   authorizePath: lazyCapabilities(apiPath`sys/control-group/authorize`),
   canAuthorize: alias('authorizePath.canUpdate'),
