@@ -1,10 +1,14 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
+import { attr } from '@ember-data/model';
 import { computed } from '@ember/object';
-import DS from 'ember-data';
 import IdentityModel from './_base';
-const { attr } = DS;
 
 export default IdentityModel.extend({
-  formFields: computed(function() {
+  formFields: computed(function () {
     return ['toEntityId', 'fromEntityIds', 'force'];
   }),
   toEntityId: attr('string', {
