@@ -1,16 +1,19 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
+import { attr } from '@ember-data/model';
 import { computed } from '@ember/object';
-import DS from 'ember-data';
 
 import PolicyModel from './rgp';
 import { expandAttributeMeta } from 'vault/utils/field-to-attrs';
 
-let { attr } = DS;
-
 export default PolicyModel.extend({
   paths: attr({
     editType: 'stringArray',
   }),
-  additionalAttrs: computed(function() {
+  additionalAttrs: computed(function () {
     return expandAttributeMeta(this, ['enforcementLevel', 'paths']);
   }),
 });
