@@ -3,7 +3,7 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { computed } from '@ember/object';
 import { reject } from 'rsvp';
 import Route from '@ember/routing/route';
@@ -28,13 +28,15 @@ export const getManagedNamespace = (nsParam, root) => {
 };
 
 export default Route.extend(ModelBoundaryRoute, ClusterRoute, {
-  namespaceService: service('namespace'),
-  version: service(),
-  permissions: service(),
-  store: service(),
   auth: service(),
-  featureFlagService: service('featureFlag'),
   currentCluster: service(),
+  customMessages: service(),
+  flagsService: service('flags'),
+  namespaceService: service('namespace'),
+  permissions: service(),
+  router: service(),
+  store: service(),
+  version: service(),
   modelTypes: computed(function () {
     return ['node', 'secret', 'secret-engine'];
   }),
@@ -47,19 +49,24 @@ export default Route.extend(ModelBoundaryRoute, ClusterRoute, {
 
   getClusterId(params) {
     const { cluster_name } = params;
-    const cluster = this.modelFor('vault').findBy('name', cluster_name);
-    return cluster ? cluster.get('id') : null;
+    const records = this.store.peekAll('cluster');
+    const cluster = records.find((record) => record.name === cluster_name);
+    return cluster?.id ?? null;
   },
 
   async beforeModel() {
     const params = this.paramsFor(this.routeName);
     let namespace = params.namespaceQueryParam;
-    const currentTokenName = this.auth.get('currentTokenName');
-    const managedRoot = this.featureFlagService.managedNamespaceRoot;
+    const currentTokenName = this.auth.currentTokenName;
+    const managedRoot = this.flagsService.hvdManagedNamespaceRoot;
     assert(
       'Cannot use VAULT_CLOUD_ADMIN_NAMESPACE flag with non-enterprise Vault version',
-      !(managedRoot && this.version.isOSS)
+      !(managedRoot && this.version.isCommunity)
     );
+
+    // activatedFlags are called this high in routing to return a response used to show/hide Secrets sync on sidebar nav.
+    await this.flagsService.fetchActivatedFlags();
+
     if (!namespace && currentTokenName && !Ember.testing) {
       // if no namespace queryParam and user authenticated,
       // use user's root namespace to redirect to properly param'd url
@@ -67,12 +74,12 @@ export default Route.extend(ModelBoundaryRoute, ClusterRoute, {
       namespace = storage?.userRootNamespace;
       // only redirect if something other than nothing
       if (namespace) {
-        this.transitionTo({ queryParams: { namespace } });
+        this.router.transitionTo({ queryParams: { namespace } });
       }
     } else if (managedRoot !== null) {
       const managed = getManagedNamespace(namespace, managedRoot);
       if (managed !== namespace) {
-        this.transitionTo({ queryParams: { namespace: managed } });
+        this.router.transitionTo({ queryParams: { namespace: managed } });
       }
     }
     this.namespaceService.setNamespace(namespace);
@@ -80,6 +87,7 @@ export default Route.extend(ModelBoundaryRoute, ClusterRoute, {
     if (id) {
       this.auth.setCluster(id);
       if (this.auth.currentToken) {
+        this.version.fetchVersion();
         await this.permissions.getPaths.perform();
       }
       return this.version.fetchFeatures();
@@ -119,11 +127,15 @@ export default Route.extend(ModelBoundaryRoute, ClusterRoute, {
   afterModel(model, transition) {
     this._super(...arguments);
     this.currentCluster.setCluster(model);
+    if (model.needsInit && this.auth.currentToken) {
+      // clear token to prevent infinite load state
+      this.auth.deleteCurrentToken();
+    }
 
     // Check that namespaces is enabled and if not,
     // clear the namespace by transition to this route w/o it
     if (this.namespaceService.path && !this.version.hasNamespaces) {
-      return this.transitionTo(this.routeName, { queryParams: { namespace: '' } });
+      return this.router.transitionTo(this.routeName, { queryParams: { namespace: '' } });
     }
     return this.transitionToTargetRoute(transition);
   },
