@@ -6,7 +6,7 @@
 import Route from '@ember/routing/route';
 import UnsavedModelRoute from 'vault/mixins/unsaved-model-route';
 import { singularize } from 'ember-inflector';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default Route.extend(UnsavedModelRoute, {
   store: service(),
