@@ -6,7 +6,7 @@
 import Route from '@ember/routing/route';
 import UnsavedModelRoute from 'vault/mixins/unsaved-model-route';
 import { singularize } from 'ember-inflector';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default Route.extend(UnsavedModelRoute, {
   store: service(),
@@ -22,6 +22,8 @@ export default Route.extend(UnsavedModelRoute, {
   setupController(controller) {
     this._super(...arguments);
     const { item_type: itemType } = this.paramsFor('vault.cluster.access.method.item');
+    const { path: method } = this.paramsFor('vault.cluster.access.method');
     controller.set('itemType', singularize(itemType));
+    controller.set('method', method);
   },
 });
