@@ -5,7 +5,7 @@
 
 import Ember from 'ember';
 import { computed } from '@ember/object';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Route from '@ember/routing/route';
 import ModelBoundaryRoute from 'vault/mixins/model-boundary-route';
 
@@ -17,6 +17,8 @@ export default Route.extend(ModelBoundaryRoute, {
   permissions: service(),
   namespaceService: service('namespace'),
   router: service(),
+  version: service(),
+  customMessages: service(),
 
   modelTypes: computed(function () {
     return ['secret', 'secret-engine'];
@@ -32,6 +34,11 @@ export default Route.extend(ModelBoundaryRoute, {
     this.console.clearLog(true);
     this.flashMessages.clearMessages();
     this.permissions.reset();
+    this.version.version = null;
+
+    if (this.version.isEnterprise) {
+      this.customMessages.clearCustomMessages();
+    }
 
     queryParams.with = authType;
     if (ns) {
@@ -39,7 +46,7 @@ export default Route.extend(ModelBoundaryRoute, {
     }
     if (Ember.testing) {
       // Don't redirect on the test
-      this.replaceWith('vault.cluster.auth', { queryParams });
+      this.router.replaceWith('vault.cluster.auth', { queryParams });
     } else {
       const { cluster_name } = this.paramsFor('vault.cluster');
       location.assign(this.router.urlFor('vault.cluster.auth', cluster_name, { queryParams }));
