@@ -3,18 +3,19 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Route from '@ember/routing/route';
 import UnsavedModelRoute from 'vault/mixins/unsaved-model-route';
 
 export default Route.extend(UnsavedModelRoute, {
+  router: service(),
   store: service(),
   version: service(),
 
   model() {
     const policyType = this.policyType();
     if (!this.version.hasSentinel && policyType !== 'acl') {
-      return this.transitionTo('vault.cluster.policies', policyType);
+      return this.router.transitionTo('vault.cluster.policies', policyType);
     }
     return this.store.createRecord(`policy/${policyType}`, {});
   },
