@@ -1,10 +1,15 @@
-import { inject as service } from '@ember/service';
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
+import { service } from '@ember/service';
 import Base from '../cluster-route-base';
 
 export default Base.extend({
   replicationMode: service(),
   beforeModel() {
     this._super(...arguments);
-    this.get('replicationMode').setMode('dr');
+    this.replicationMode.setMode('dr');
   },
 });
