@@ -21,15 +21,34 @@ export default EditBase.extend({
     const { backend } = this.paramsFor('vault.cluster.secrets.backend');
     if (this.backendType(backend) !== 'transit') {
       if (parentKey) {
-        return this.transitionTo('vault.cluster.secrets.backend.show', parentKey);
+        return this.router.transitionTo('vault.cluster.secrets.backend.show', parentKey);
       } else {
-        return this.transitionTo('vault.cluster.secrets.backend.show-root');
+        return this.router.transitionTo('vault.cluster.secrets.backend.show-root');
       }
     }
   },
   setupController(controller, model) {
     this._super(...arguments);
     const { selectedAction } = this.paramsFor(this.routeName);
-    controller.set('selectedAction', selectedAction || model.secret.get('supportedActions.firstObject'));
+    controller.set('selectedAction', selectedAction || model.secret.supportedActions[0]);
+    controller.set('breadcrumbs', [
+      {
+        label: 'Secrets',
+        route: 'vault.cluster.secrets',
+      },
+      {
+        label: model.secret.backend,
+        route: 'vault.cluster.secrets.backend.list-root',
+        model: model.secret.backend,
+      },
+      {
+        label: model.secret.id,
+        route: 'vault.cluster.secrets.backend.show',
+        models: [model.secret.backend, model.secret.id],
+      },
+      {
+        label: 'Actions',
+      },
+    ]);
   },
 });
