@@ -4,25 +4,14 @@
  */
 
 import { hash } from 'rsvp';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import EditBase from './secret-edit';
 
 const secretModel = (store, backend, key) => {
-  const backendModel = store.peekRecord('secret-engine', backend);
-  const modelType = backendModel.modelTypeForKV;
-  if (modelType !== 'secret-v2') {
-    const model = store.createRecord(modelType, {
-      path: key,
-    });
-    return model;
-  }
-  const secret = store.createRecord(modelType);
-  secret.set('engine', backendModel);
-  const version = store.createRecord('secret-v2-version', {
+  const model = store.createRecord('secret', {
     path: key,
   });
-  secret.set('selectedVersion', version);
-  return secret;
+  return model;
 };
 
 const transformModel = (queryParams) => {
@@ -47,15 +36,9 @@ export default EditBase.extend({
     if (modelType === 'database/connection' && transition.to?.queryParams?.itemType === 'role') {
       modelType = 'database/role';
     }
-    if (modelType !== 'secret' && modelType !== 'secret-v2') {
+    if (modelType !== 'secret') {
       return this.store.createRecord(modelType);
     }
-    // create record in capabilities that checks for access to create metadata
-    // this record is then maybeQueryRecord in the component secret-create-or-update
-    if (modelType === 'secret-v2') {
-      // only check for kv2 secrets
-      this.store.findRecord('capabilities', `${backend}/metadata/`);
-    }
     return secretModel(this.store, backend, transition.to.queryParams.initialKey);
   },
 
