@@ -4,13 +4,16 @@
  */
 
 import Route from '@ember/routing/route';
+import { service } from '@ember/service';
+
+export default class SecretsBackendCreateRoute extends Route {
+  @service router;
 
-export default Route.extend({
   beforeModel() {
     const { secret, initialKey } = this.paramsFor(this.routeName);
     const qp = initialKey || secret;
-    return this.transitionTo('vault.cluster.secrets.backend.create-root', {
+    return this.router.transitionTo('vault.cluster.secrets.backend.create-root', {
       queryParams: { initialKey: qp },
     });
-  },
-});
+  }
+}
