@@ -3,18 +3,20 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Route from '@ember/routing/route';
 import { toolsActions } from 'vault/helpers/tools-actions';
 
-export default Route.extend({
-  currentCluster: service(),
+export default class ToolsIndexRouter extends Route {
+  @service currentCluster;
+  @service router;
+
   beforeModel(transition) {
     const currentCluster = this.currentCluster.cluster.name;
     const supportedActions = toolsActions();
     if (transition.targetName === this.routeName) {
       transition.abort();
-      return this.replaceWith('vault.cluster.tools.tool', currentCluster, supportedActions[0]);
+      return this.router.replaceWith('vault.cluster.tools.tool', currentCluster, supportedActions[0]);
     }
-  },
-});
+  }
+}
