@@ -4,7 +4,6 @@
  */
 
 import RESTSerializer from '@ember-data/serializer/rest';
-import { assign } from '@ember/polyfills';
 import { decamelize } from '@ember/string';
 
 export default RESTSerializer.extend({
@@ -14,7 +13,7 @@ export default RESTSerializer.extend({
 
   normalizeAll(payload) {
     if (payload.data) {
-      const data = assign({}, payload, payload.data);
+      const data = { ...payload, ...payload.data };
       return [data];
     }
     return [payload];
