@@ -3,6 +3,6 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import CertSerializer from './cert';
+import CertSerializer from './certificate/base';
 
 export default class PkiCertificateSerializer extends CertSerializer {}
