@@ -3,43 +3,56 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { alias, equal } from '@ember/object/computed';
-import Service, { inject as service } from '@ember/service';
+import Service, { service } from '@ember/service';
 import { task } from 'ember-concurrency';
-import { computed } from '@ember/object';
 import { getRelativePath } from 'core/utils/sanitize-path';
+import { tracked } from '@glimmer/tracking';
 import { buildWaiter } from '@ember/test-waiters';
 
 const waiter = buildWaiter('namespaces');
 const ROOT_NAMESPACE = '';
-export default Service.extend({
-  store: service(),
-  auth: service(),
-  userRootNamespace: alias('auth.authData.userRootNamespace'),
+export default class NamespaceService extends Service {
+  @service store;
+  @service auth;
+
   //populated by the query param on the cluster route
-  path: '',
+  @tracked path = '';
   // list of namespaces available to the current user under the
   // current namespace
-  accessibleNamespaces: null,
+  @tracked accessibleNamespaces = null;
 
-  inRootNamespace: equal('path', ROOT_NAMESPACE),
+  get userRootNamespace() {
+    return this.auth.authData?.userRootNamespace;
+  }
 
-  relativeNamespace: computed('path', 'userRootNamespace', function () {
+  get inRootNamespace() {
+    return this.path === ROOT_NAMESPACE;
+  }
+
+  get currentNamespace() {
+    if (this.inRootNamespace) return 'root';
+
+    const parts = this.path?.split('/');
+    return parts[parts.length - 1];
+  }
+
+  get relativeNamespace() {
     // relative namespace is the current namespace minus the user's root.
     // so if we're in app/staging/group1 but the user's root is app, the
     // relative namespace is staging/group
     return getRelativePath(this.path, this.userRootNamespace);
-  }),
+  }
 
   setNamespace(path) {
     if (!path) {
-      this.set('path', '');
+      this.path = '';
       return;
     }
-    this.set('path', path);
-  },
+    this.path = path;
+  }
 
-  findNamespacesForUser: task(function* () {
+  @task({ drop: true })
+  *findNamespacesForUser() {
     const waiterToken = waiter.beginAsync();
     // uses the adapter and the raw response here since
     // models get wiped when switching namespaces and we
@@ -55,28 +68,25 @@ export default Service.extend({
         },
       });
       const keys = ns.data.keys || [];
-      this.set(
-        'accessibleNamespaces',
-        keys.map((n) => {
-          let fullNS = n;
-          // if the user's root isn't '', then we need to construct
-          // the paths so they connect to the user root to the list
-          // otherwise using the current ns to grab the correct leaf
-          // node in the graph doesn't work
-          if (userRoot) {
-            fullNS = `${userRoot}/${n}`;
-          }
-          return fullNS.replace(/\/$/, '');
-        })
-      );
+      this.accessibleNamespaces = keys.map((n) => {
+        let fullNS = n;
+        // if the user's root isn't '', then we need to construct
+        // the paths so they connect to the user root to the list
+        // otherwise using the current ns to grab the correct leaf
+        // node in the graph doesn't work
+        if (userRoot) {
+          fullNS = `${userRoot}/${n}`;
+        }
+        return fullNS.replace(/\/$/, '');
+      });
     } catch (e) {
       //do nothing here
     } finally {
       waiter.endAsync(waiterToken);
     }
-  }).drop(),
+  }
 
   reset() {
-    this.set('accessibleNamespaces', null);
-  },
-});
+    this.accessibleNamespaces = null;
+  }
+}
