@@ -12,9 +12,8 @@ import { encodePath } from 'vault/utils/path-encoding-helpers';
  * KeyValueHeader components show breadcrumbs for secret engines.
  *
  * @example
- * ```js
- <KeyValueHeader @path="vault.cluster.secrets.backend.show" @mode={{this.mode}} @root={{@root}}/>
- * ```
+ * <KeyValueHeader @path="vault.cluster.secrets.backend.show" @mode={{this.mode}}/>
+ *
  * @param {string} [mode=null] - Used to set the currentPath.
  * @param {string} [baseKey=null] - Used to generate the path backward.
  * @param {string} [path=null] - The fallback path.
