@@ -3,7 +3,7 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { equal } from '@ember/object/computed';
 import { get, computed } from '@ember/object';
 import Component from '@ember/component';
