@@ -4,7 +4,7 @@
  */
 
 /* eslint-disable ember/no-observers */
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { isArray } from '@ember/array';
 import Helper from '@ember/component/helper';
 import { observer } from '@ember/object';
@@ -21,8 +21,8 @@ export default Helper.extend({
 
   compute([routeName, model], { isExact }) {
     const router = this.router;
-    const currentRoute = router.get('currentRouteName');
-    let currentURL = router.get('currentURL');
+    const currentRoute = router.currentRouteName;
+    let currentURL = router.currentURL;
     // if we have any query params we want to discard them
     currentURL = currentURL?.split('?')[0];
     const comparator = isExact ? exact : startsWith;
