@@ -3,7 +3,7 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { or } from '@ember/object/computed';
 import { isPresent } from '@ember/utils';
 import Mixin from '@ember/object/mixin';
@@ -13,7 +13,6 @@ export default Mixin.create({
   store: service(),
   router: service(),
   loading: or('save.isRunning', 'submitSuccess.isRunning'),
-  onEnable() {},
   onDisable() {},
   onPromote() {},
   submitHandler: task(function* (action, clusterMode, data, event) {
@@ -53,10 +52,9 @@ export default Mixin.create({
     return yield this.submitSuccess.perform(resp, action, clusterMode);
   }).drop(),
 
-  submitSuccess: task(function* (resp, action, mode) {
+  submitSuccess: task(function* (resp, action) {
+    // enable action is handled separately in EnableReplicationForm component
     const cluster = this.cluster;
-    const replicationMode = this.selectedReplicationMode || this.replicationMode;
-    const store = this.store;
     if (!cluster) {
       return;
     }
@@ -75,20 +73,6 @@ export default Mixin.create({
     if (this.reset) {
       this.reset();
     }
-    if (action === 'enable') {
-      // do something to show model is pending
-      cluster.set(
-        replicationMode,
-        store.createRecord('replication-attributes', {
-          mode: 'bootstrapping',
-        })
-      );
-      if (mode === 'secondary' && replicationMode === 'performance') {
-        // if we're enabing a secondary, there could be mount filtering,
-        // so we should unload all of the backends
-        store.unloadAll('secret-engine');
-      }
-    }
     try {
       yield cluster.reload();
     } catch (e) {
@@ -101,11 +85,6 @@ export default Mixin.create({
     if (action === 'promote') {
       yield this.onPromote();
     }
-    if (action === 'enable') {
-      /// onEnable is a method available only to route vault.cluster.replication.index
-      // if action 'enable' is called from vault.cluster.replication.mode.index this method is not called
-      yield this.onEnable(replicationMode, mode);
-    }
   }).drop(),
 
   submitError(e) {
