@@ -1,9 +1,14 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 // adapted from https://gist.github.com/jed/964849
-let fn = (function(anchor) {
-  return function(url) {
+const fn = (function (anchor) {
+  return function (url) {
     anchor.href = url;
-    let parts = {};
-    for (let prop in anchor) {
+    const parts = {};
+    for (const prop in anchor) {
       if ('' + anchor[prop] === anchor[prop]) {
         parts[prop] = anchor[prop];
       }
