@@ -3,4 +3,4 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-export { default, isActiveRoute } from 'core/helpers/is-active-route';
+export { default } from 'core/helpers/is-active-route';
