@@ -3,4 +3,4 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-export { default, setFlashMessage } from 'core/helpers/set-flash-message';
+export { default } from 'core/helpers/set-flash-message';
