@@ -3,4 +3,4 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-export { default } from 'core/helpers/to-label';
+export { default, toLabel } from 'core/helpers/to-label';
