@@ -4,7 +4,7 @@
  */
 
 import Route from '@ember/routing/route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default class KubernetesRolesCreateRoute extends Route {
   @service store;
@@ -20,8 +20,8 @@ export default class KubernetesRolesCreateRoute extends Route {
 
     controller.breadcrumbs = [
       { label: resolvedModel.backend, route: 'overview' },
-      { label: 'roles', route: 'roles' },
-      { label: 'create' },
+      { label: 'Roles', route: 'roles', model: resolvedModel.backend },
+      { label: 'Create' },
     ];
   }
 }
