@@ -17,15 +17,17 @@ export default class KvEngine extends Engine {
   Resolver = Resolver;
   dependencies = {
     services: [
+      'capabilities',
+      'control-group',
       'download',
+      'flash-messages',
       'namespace',
       'router',
-      'store',
       'secret-mount-path',
-      'flash-messages',
-      'control-group',
+      'store',
+      'version',
     ],
-    externalRoutes: ['secrets'],
+    externalRoutes: ['secrets', 'syncDestination'],
   };
 }
 
