@@ -4,7 +4,7 @@
  */
 
 import Route from '@ember/routing/route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import { hash } from 'rsvp';
 
 export default class KvConfigurationRoute extends Route {
@@ -24,9 +24,9 @@ export default class KvConfigurationRoute extends Route {
   setupController(controller, resolvedModel) {
     super.setupController(controller, resolvedModel);
     controller.breadcrumbs = [
-      { label: 'secrets', route: 'secrets', linkExternal: true },
-      { label: resolvedModel.mountConfig.id, route: 'list' },
-      { label: 'configuration' },
+      { label: 'Secrets', route: 'secrets', linkExternal: true },
+      { label: resolvedModel.mountConfig.id, route: 'list', model: resolvedModel.engineConfig.backend },
+      { label: 'Configuration' },
     ];
   }
 }
