@@ -11,11 +11,10 @@ export default class KvSecretDetailsIndexRoute extends Route {
     super.setupController(controller, resolvedModel);
 
     const breadcrumbsArray = [
-      { label: 'secrets', route: 'secrets', linkExternal: true },
-      { label: resolvedModel.backend, route: 'list' },
-      ...breadcrumbsForSecret(resolvedModel.path, true),
+      { label: 'Secrets', route: 'secrets', linkExternal: true },
+      { label: resolvedModel.backend, route: 'list', model: resolvedModel.backend },
+      ...breadcrumbsForSecret(resolvedModel.backend, resolvedModel.path, true),
     ];
-
     controller.breadcrumbs = breadcrumbsArray;
   }
 }
