@@ -3,4 +3,4 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-export { default } from 'kv/utils/kv-deleted';
+export { isDeleted as default } from 'kv/utils/kv-deleted';
