@@ -16,6 +16,10 @@ module.exports = buildEngine({
     enabled: false,
   },
 
+  babel: {
+    plugins: [require.resolve('ember-concurrency/async-arrow-task-transform')],
+  },
+
   isDevelopingAddon() {
     return true;
   },
