@@ -4,7 +4,7 @@
  */
 
 import Route from '@ember/routing/route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 import type Transition from '@ember/routing/transition';
 import type AdapterError from 'ember-data/adapter'; // eslint-disable-line ember/use-ember-data-rfc-395-imports
@@ -24,7 +24,7 @@ export default class LdapErrorRoute extends Route {
   setupController(controller: LdapErrorController, resolvedModel: AdapterError, transition: Transition) {
     super.setupController(controller, resolvedModel, transition);
     controller.breadcrumbs = [
-      { label: 'secrets', route: 'secrets', linkExternal: true },
+      { label: 'Secrets', route: 'secrets', linkExternal: true },
       { label: this.secretMountPath.currentPath, route: 'overview' },
     ];
     controller.backend = this.modelFor('application') as SecretEngineModel;
