@@ -12,12 +12,14 @@ export default class TidyAutoIndexRoute extends Route {
 
   // inherits model from tidy/auto
 
-  setupController(controller) {
+  setupController(controller, resolvedModel) {
+    // autoTidyConfig id is the backend path
+    const { id: backend } = resolvedModel;
     super.setupController(...arguments);
     controller.breadcrumbs = [
-      { label: 'secrets', route: 'secrets', linkExternal: true },
-      { label: this.secretMountPath.currentPath, route: 'overview' },
-      { label: 'tidy', route: 'tidy.index' },
+      { label: 'Secrets', route: 'secrets', linkExternal: true },
+      { label: this.secretMountPath.currentPath, route: 'overview', model: backend },
+      { label: 'tidy', route: 'tidy.index', model: backend },
       { label: 'auto' },
     ];
     controller.title = this.secretMountPath.currentPath;
