@@ -1,9 +1,15 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import { setProperties } from '@ember/object';
 import { hash } from 'rsvp';
 import Route from '@ember/routing/route';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 
 export default Route.extend({
+  router: service(),
   store: service(),
   model() {
     const replicationMode = this.paramsFor('mode').replication_mode;
@@ -12,10 +18,10 @@ export default Route.extend({
       cluster: this.modelFor('mode'),
       canAddSecondary: this.store
         .findRecord('capabilities', `sys/replication/${replicationMode}/primary/secondary-token`)
-        .then(c => c.get('canUpdate')),
+        .then((c) => c.canUpdate),
       canRevokeSecondary: this.store
         .findRecord('capabilities', `sys/replication/${replicationMode}/primary/revoke-secondary`)
-        .then(c => c.get('canUpdate')),
+        .then((c) => c.canUpdate),
     }).then(({ cluster, canAddSecondary, canRevokeSecondary }) => {
       setProperties(cluster, {
         canRevokeSecondary,
@@ -26,12 +32,9 @@ export default Route.extend({
   },
   afterModel(model) {
     const replicationMode = this.paramsFor('mode').replication_mode;
-    if (
-      !model.get(`${replicationMode}.isPrimary`) ||
-      model.get(`${replicationMode}.replicationDisabled`) ||
-      model.get(`${replicationMode}.replicationUnsupported`)
-    ) {
-      return this.transitionTo('mode', replicationMode);
+    const modeModel = model[replicationMode];
+    if (!modeModel.isPrimary || modeModel.replicationDisabled || modeModel.replicationUnsupported) {
+      this.router.transitionTo('vault.cluster.replication.mode', replicationMode);
     }
   },
 });
