@@ -4,11 +4,12 @@
  */
 
 import { alias } from '@ember/object/computed';
-import { inject as service } from '@ember/service';
+import { service } from '@ember/service';
 import Route from '@ember/routing/route';
 import UnloadModelRouteMixin from 'vault/mixins/unload-model-route';
 
 export default Route.extend(UnloadModelRouteMixin, {
+  router: service(),
   store: service(),
   version: service(),
   rm: service('replication-mode'),
