@@ -7,15 +7,19 @@
 // individual lookup done in mirage config
 import base from './base';
 import chrootNamespace from './chroot-namespace';
+import customMessages from './custom-messages';
 import clients from './clients';
 import database from './database';
+import hcpLink from './hcp-link';
 import kms from './kms';
+import kubernetes from './kubernetes';
+import ldap from './ldap';
 import mfaConfig from './mfa-config';
 import mfaLogin from './mfa-login';
 import oidcConfig from './oidc-config';
-import hcpLink from './hcp-link';
-import kubernetes from './kubernetes';
-import ldap from './ldap';
+import reducedDisclosure from './reduced-disclosure';
+import sync from './sync';
+import replication from './replication';
 
 export {
   base,
@@ -29,4 +33,8 @@ export {
   mfaConfig,
   mfaLogin,
   oidcConfig,
+  reducedDisclosure,
+  customMessages,
+  sync,
+  replication,
 };
