@@ -3,22 +3,12 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
+import { waitFor } from '@ember/test-helpers';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
-
+import { setRunOptions } from 'ember-a11y-testing/test-support';
+import { setupMirage } from 'ember-cli-mirage/test-support';
 import initPage from 'vault/tests/pages/init';
-import Pretender from 'pretender';
-
-const HEALTH_RESPONSE = {
-  initialized: false,
-  sealed: true,
-  standby: true,
-  performance_standby: false,
-  replication_performance_mode: 'unknown',
-  replication_dr_mode: 'unknown',
-  server_time_utc: 1538066726,
-  version: '1.13.0-dev1',
-};
 
 const CLOUD_SEAL_RESPONSE = {
   keys: [],
@@ -82,56 +72,48 @@ const assertRequest = (req, assert, isCloud) => {
 
 module('Acceptance | init', function (hooks) {
   setupApplicationTest(hooks);
-
-  const setInitResponse = (server, resp) => {
-    server.put('/v1/sys/init', () => {
-      return [200, { 'Content-Type': 'application/json' }, JSON.stringify(resp)];
-    });
-  };
-  const setStatusResponse = (server, resp) => {
-    server.get('/v1/sys/seal-status', () => {
-      return [200, { 'Content-Type': 'application/json' }, JSON.stringify(resp)];
-    });
-  };
-  hooks.beforeEach(function () {
-    this.server = new Pretender();
-    this.server.get('/v1/sys/health', () => {
-      return [200, { 'Content-Type': 'application/json' }, JSON.stringify(HEALTH_RESPONSE)];
-    });
-    this.server.get('/v1/sys/internal/ui/feature-flags', this.server.passthrough);
-  });
-
-  hooks.afterEach(function () {
-    this.server.shutdown();
-  });
+  setupMirage(hooks);
 
   test('cloud seal init', async function (assert) {
+    // continue button is disabled, violating color-contrast
+    setRunOptions({
+      rules: {
+        'color-contrast': { enabled: false },
+      },
+    });
     assert.expect(6);
-
-    setInitResponse(this.server, CLOUD_SEAL_RESPONSE);
-    setStatusResponse(this.server, CLOUD_SEAL_STATUS_RESPONSE);
+    this.server.put('/sys/init', (schema, req) => {
+      assertRequest(req, assert, true);
+      return CLOUD_SEAL_RESPONSE;
+    });
+    this.server.get('/sys/seal-status', () => {
+      return CLOUD_SEAL_STATUS_RESPONSE;
+    });
 
     await initPage.init(5, 3);
-
+    await waitFor('[data-test-advance-button]');
     assert.strictEqual(
       initPage.keys.length,
       CLOUD_SEAL_RESPONSE.recovery_keys.length,
       'shows all of the recovery keys'
     );
     assert.strictEqual(initPage.buttonText, 'Continue to Authenticate', 'links to authenticate');
-    assertRequest(this.server.handledRequests.findBy('url', '/v1/sys/init'), assert, true);
   });
 
   test('shamir seal init', async function (assert) {
     assert.expect(6);
 
-    setInitResponse(this.server, SEAL_RESPONSE);
-    setStatusResponse(this.server, SEAL_STATUS_RESPONSE);
+    this.server.put('/sys/init', (schema, req) => {
+      assertRequest(req, assert, false);
+      return SEAL_RESPONSE;
+    });
+    this.server.get('/sys/seal-status', () => {
+      return SEAL_STATUS_RESPONSE;
+    });
 
     await initPage.init(3, 2);
-
+    await waitFor('[data-test-advance-button]');
     assert.strictEqual(initPage.keys.length, SEAL_RESPONSE.keys.length, 'shows all of the recovery keys');
     assert.strictEqual(initPage.buttonText, 'Continue to Unseal', 'links to unseal');
-    assertRequest(this.server.handledRequests.findBy('url', '/v1/sys/init'), assert, false);
   });
 });
