@@ -11,11 +11,20 @@ import authPage from 'vault/tests/pages/auth';
 import logout from 'vault/tests/pages/logout';
 import enablePage from 'vault/tests/pages/settings/mount-secret-backend';
 import { click, currentURL, fillIn, visit } from '@ember/test-helpers';
-import { runCommands } from 'vault/tests/helpers/pki/pki-run-commands';
-import { SELECTORS } from 'vault/tests/helpers/pki/workflow';
+import { runCmd } from 'vault/tests/helpers/commands';
+import { GENERAL } from 'vault/tests/helpers/general-selectors';
+import {
+  PKI_CONFIGURE_CREATE,
+  PKI_ISSUER_DETAILS,
+  PKI_ISSUER_LIST,
+  PKI_KEYS,
+  PKI_ROLE_DETAILS,
+} from 'vault/tests/helpers/pki/pki-selectors';
 
+const OVERVIEW_BREADCRUMB = '[data-test-breadcrumbs] li:nth-of-type(2) > a';
 /**
- * This test module should test that dirty route models are cleaned up when the user leaves the page
+ * This test module should test that dirty route models are cleaned up when
+ * the user leaves the page via cancel or breadcrumb navigation
  */
 module('Acceptance | pki engine route cleanup test', function (hooks) {
   setupApplicationTest(hooks);
@@ -34,7 +43,7 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
     await logout.visit();
     await authPage.login();
     // Cleanup engine
-    await runCommands([`delete sys/mounts/${this.mountPath}`]);
+    await runCmd([`delete sys/mounts/${this.mountPath}`]);
   });
 
   module('configuration', function () {
@@ -42,31 +51,31 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let configs, urls, config;
       await authPage.login(this.pkiAdminToken);
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.emptyStateLink);
+      await click(`${GENERAL.emptyStateActions} a`);
       configs = this.store.peekAll('pki/action');
       urls = this.store.peekRecord('pki/config/urls', this.mountPath);
-      config = configs.objectAt(0);
+      config = configs.at(0);
       assert.strictEqual(configs.length, 1, 'One config model present');
       assert.false(urls.hasDirtyAttributes, 'URLs is loaded from endpoint');
       assert.true(config.hasDirtyAttributes, 'Config model is dirty');
 
       // Cancel button rolls it back
-      await click(SELECTORS.configuration.cancelButton);
+      await click(GENERAL.cancelButton);
       configs = this.store.peekAll('pki/action');
       urls = this.store.peekRecord('pki/config/urls', this.mountPath);
       assert.strictEqual(configs.length, 0, 'config model is rolled back on cancel');
       assert.strictEqual(urls.id, this.mountPath, 'Urls still exists on exit');
 
-      await click(SELECTORS.emptyStateLink);
+      await click(`${GENERAL.emptyStateActions} a`);
       configs = this.store.peekAll('pki/action');
       urls = this.store.peekRecord('pki/config/urls', this.mountPath);
-      config = configs.objectAt(0);
+      config = configs.at(0);
       assert.strictEqual(configs.length, 1, 'One config model present');
       assert.false(urls.hasDirtyAttributes, 'URLs is loaded from endpoint');
       assert.true(config.hasDirtyAttributes, 'Config model is dirty');
 
       // Exit page via link rolls it back
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       configs = this.store.peekAll('pki/action');
       urls = this.store.peekRecord('pki/config/urls', this.mountPath);
       assert.strictEqual(configs.length, 0, 'config model is rolled back on cancel');
@@ -79,11 +88,11 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       await authPage.login();
       // Configure PKI
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.emptyStateLink);
-      await click(SELECTORS.configuration.optionByKey('generate-root'));
-      await fillIn(SELECTORS.configuration.typeField, 'internal');
-      await fillIn(SELECTORS.configuration.inputByName('commonName'), 'my-root-cert');
-      await click(SELECTORS.configuration.generateRootSave);
+      await click(`${GENERAL.emptyStateActions} a`);
+      await click(PKI_CONFIGURE_CREATE.optionByKey('generate-root'));
+      await fillIn(GENERAL.inputByAttr('type'), 'internal');
+      await fillIn(GENERAL.inputByAttr('commonName'), 'my-root-cert');
+      await click(GENERAL.saveButton);
       await logout.visit();
     });
 
@@ -92,15 +101,15 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       await authPage.login();
       // Create PKI
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.rolesTab);
+      await click(GENERAL.secretTab('Roles'));
       roles = this.store.peekAll('pki/role');
       assert.strictEqual(roles.length, 0, 'No roles exist yet');
-      await click(SELECTORS.createRoleLink);
+      await click(PKI_ROLE_DETAILS.createRoleLink);
       roles = this.store.peekAll('pki/role');
-      const role = roles.objectAt(0);
+      const role = roles.at(0);
       assert.strictEqual(roles.length, 1, 'New role exists');
       assert.true(role.isNew, 'Role is new model');
-      await click(SELECTORS.roleForm.roleCancelButton);
+      await click(GENERAL.cancelButton);
       roles = this.store.peekAll('pki/role');
       assert.strictEqual(roles.length, 0, 'Role is removed from store');
     });
@@ -109,15 +118,15 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       await authPage.login();
       // Create PKI
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.rolesTab);
+      await click(GENERAL.secretTab('Roles'));
       roles = this.store.peekAll('pki/role');
       assert.strictEqual(roles.length, 0, 'No roles exist yet');
-      await click(SELECTORS.createRoleLink);
+      await click(PKI_ROLE_DETAILS.createRoleLink);
       roles = this.store.peekAll('pki/role');
-      const role = roles.objectAt(0);
+      const role = roles.at(0);
       assert.strictEqual(roles.length, 1, 'New role exists');
       assert.true(role.isNew, 'Role is new model');
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       roles = this.store.peekAll('pki/role');
       assert.strictEqual(roles.length, 0, 'Role is removed from store');
     });
@@ -127,38 +136,38 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       await authPage.login();
       // Create PKI
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.rolesTab);
+      await click(GENERAL.secretTab('Roles'));
       roles = this.store.peekAll('pki/role');
       assert.strictEqual(roles.length, 0, 'No roles exist yet');
-      await click(SELECTORS.createRoleLink);
-      await fillIn(SELECTORS.roleForm.roleName, roleId);
-      await click(SELECTORS.roleForm.roleCreateButton);
-      assert.dom('[data-test-value-div="Role name"]').hasText(roleId, 'Shows correct role after create');
+      await click(PKI_ROLE_DETAILS.createRoleLink);
+      await fillIn(GENERAL.inputByAttr('name'), roleId);
+      await click(GENERAL.saveButton);
+      assert.dom(GENERAL.infoRowValue('Role name')).hasText(roleId, 'Shows correct role after create');
       roles = this.store.peekAll('pki/role');
-      role = roles.objectAt(0);
+      role = roles.at(0);
       assert.strictEqual(roles.length, 1, 'Role is created');
       assert.false(role.hasDirtyAttributes, 'Role no longer has dirty attributes');
 
       // Edit role
-      await click(SELECTORS.editRoleLink);
-      await click(SELECTORS.roleForm.issuerRefToggle);
-      await fillIn(SELECTORS.roleForm.issuerRefSelect, 'foobar');
+      await click(PKI_ROLE_DETAILS.editRoleLink);
+      await click(GENERAL.ttl.toggle('issuerRef-toggle'));
+      await fillIn(GENERAL.selectByAttr('issuerRef'), 'foobar');
       role = this.store.peekRecord('pki/role', roleId);
       assert.true(role.hasDirtyAttributes, 'Role has dirty attrs');
       // Exit page via cancel button
-      await click(SELECTORS.roleForm.roleCancelButton);
+      await click(GENERAL.cancelButton);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/roles/${roleId}/details`);
       role = this.store.peekRecord('pki/role', roleId);
       assert.false(role.hasDirtyAttributes, 'Role dirty attrs have been rolled back');
 
       // Edit again
-      await click(SELECTORS.editRoleLink);
-      await click(SELECTORS.roleForm.issuerRefToggle);
-      await fillIn(SELECTORS.roleForm.issuerRefSelect, 'foobar2');
+      await click(PKI_ROLE_DETAILS.editRoleLink);
+      await click(GENERAL.ttl.toggle('issuerRef-toggle'));
+      await fillIn(GENERAL.selectByAttr('issuerRef'), 'foobar2');
       role = this.store.peekRecord('pki/role', roleId);
       assert.true(role.hasDirtyAttributes, 'Role has dirty attrs');
       // Exit page via breadcrumbs
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       role = this.store.peekRecord('pki/role', roleId);
       assert.false(role.hasDirtyAttributes, 'Role dirty attrs have been rolled back');
     });
@@ -169,13 +178,13 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let issuers;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.issuersTab);
+      await click(GENERAL.secretTab('Issuers'));
       issuers = this.store.peekAll('pki/issuer');
       assert.strictEqual(issuers.length, 0, 'No issuer models exist yet');
-      await click(SELECTORS.importIssuerLink);
+      await click(PKI_ISSUER_LIST.importIssuerLink);
       issuers = this.store.peekAll('pki/action');
       assert.strictEqual(issuers.length, 1, 'Action model created');
-      const issuer = issuers.objectAt(0);
+      const issuer = issuers.at(0);
       assert.true(issuer.hasDirtyAttributes, 'Action has dirty attrs');
       assert.true(issuer.isNew, 'Action is new');
       // Exit
@@ -188,17 +197,17 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let issuers;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.issuersTab);
+      await click(GENERAL.secretTab('Issuers'));
       issuers = this.store.peekAll('pki/issuer');
       assert.strictEqual(issuers.length, 0, 'No issuers exist yet');
-      await click(SELECTORS.importIssuerLink);
+      await click(PKI_ISSUER_LIST.importIssuerLink);
       issuers = this.store.peekAll('pki/action');
       assert.strictEqual(issuers.length, 1, 'Action model created');
-      const issuer = issuers.objectAt(0);
+      const issuer = issuers.at(0);
       assert.true(issuer.hasDirtyAttributes, 'Action model has dirty attrs');
       assert.true(issuer.isNew, 'Action model is new');
       // Exit
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/overview`);
       issuers = this.store.peekAll('pki/action');
       assert.strictEqual(issuers.length, 0, 'Issuer is removed from store');
@@ -207,19 +216,19 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let actions;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.issuersTab);
+      await click(GENERAL.secretTab('Issuers'));
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 0, 'No actions exist yet');
-      await click(SELECTORS.generateIssuerDropdown);
-      await click(SELECTORS.generateIssuerRoot);
+      await click(PKI_ISSUER_LIST.generateIssuerDropdown);
+      await click(PKI_ISSUER_LIST.generateIssuerRoot);
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 1, 'Action model for generate-root created');
-      const action = actions.objectAt(0);
+      const action = actions.at(0);
       assert.true(action.hasDirtyAttributes, 'Action has dirty attrs');
       assert.true(action.isNew, 'Action is new');
       assert.strictEqual(action.actionType, 'generate-root', 'Action type is correct');
       // Exit
-      await click(SELECTORS.configuration.generateRootCancel);
+      await click(GENERAL.cancelButton);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/issuers`);
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 0, 'Action is removed from store');
@@ -228,19 +237,19 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let actions;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.issuersTab);
+      await click(GENERAL.secretTab('Issuers'));
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 0, 'No actions exist yet');
-      await click(SELECTORS.generateIssuerDropdown);
-      await click(SELECTORS.generateIssuerRoot);
+      await click(PKI_ISSUER_LIST.generateIssuerDropdown);
+      await click(PKI_ISSUER_LIST.generateIssuerRoot);
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 1, 'Action model for generate-root created');
-      const action = actions.objectAt(0);
+      const action = actions.at(0);
       assert.true(action.hasDirtyAttributes, 'Action has dirty attrs');
       assert.true(action.isNew, 'Action is new');
       assert.strictEqual(action.actionType, 'generate-root');
       // Exit
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/overview`);
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 0, 'Action is removed from store');
@@ -249,14 +258,14 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let actions;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.issuersTab);
+      await click(GENERAL.secretTab('Issuers'));
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 0, 'No actions exist yet');
-      await await click(SELECTORS.generateIssuerDropdown);
-      await click(SELECTORS.generateIssuerIntermediate);
+      await await click(PKI_ISSUER_LIST.generateIssuerDropdown);
+      await click(PKI_ISSUER_LIST.generateIssuerIntermediate);
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 1, 'Action model for generate-csr created');
-      const action = actions.objectAt(0);
+      const action = actions.at(0);
       assert.true(action.hasDirtyAttributes, 'Action has dirty attrs');
       assert.true(action.isNew, 'Action is new');
       assert.strictEqual(action.actionType, 'generate-csr');
@@ -270,19 +279,19 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let actions;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.issuersTab);
+      await click(GENERAL.secretTab('Issuers'));
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 0, 'No actions exist yet');
-      await click(SELECTORS.generateIssuerDropdown);
-      await click(SELECTORS.generateIssuerIntermediate);
+      await click(PKI_ISSUER_LIST.generateIssuerDropdown);
+      await click(PKI_ISSUER_LIST.generateIssuerIntermediate);
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 1, 'Action model for generate-csr created');
-      const action = actions.objectAt(0);
+      const action = actions.at(0);
       assert.true(action.hasDirtyAttributes, 'Action has dirty attrs');
       assert.true(action.isNew, 'Action is new');
       assert.strictEqual(action.actionType, 'generate-csr');
       // Exit
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/overview`);
       actions = this.store.peekAll('pki/action');
       assert.strictEqual(actions.length, 0, 'Action is removed from store');
@@ -291,24 +300,24 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let issuers, issuer;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.emptyStateLink);
-      await click(SELECTORS.configuration.optionByKey('generate-root'));
-      await fillIn(SELECTORS.configuration.typeField, 'internal');
-      await fillIn(SELECTORS.configuration.inputByName('commonName'), 'my-root-cert');
-      await click(SELECTORS.configuration.generateRootSave);
+      await click(`${GENERAL.emptyStateActions} a`);
+      await click(PKI_CONFIGURE_CREATE.optionByKey('generate-root'));
+      await fillIn(GENERAL.inputByAttr('type'), 'internal');
+      await fillIn(GENERAL.inputByAttr('commonName'), 'my-root-cert');
+      await click(GENERAL.saveButton);
       // Go to list view so we fetch all the issuers
       await visit(`/vault/secrets/${this.mountPath}/pki/issuers`);
 
       issuers = this.store.peekAll('pki/issuer');
-      const issuerId = issuers.objectAt(0).id;
+      const issuerId = issuers.at(0).id;
       assert.strictEqual(issuers.length, 1, 'Issuer exists on model in list');
       await visit(`/vault/secrets/${this.mountPath}/pki/issuers/${issuerId}/details`);
-      await click(SELECTORS.issuerDetails.configure);
+      await click(PKI_ISSUER_DETAILS.configure);
       issuer = this.store.peekRecord('pki/issuer', issuerId);
       assert.false(issuer.hasDirtyAttributes, 'Model not dirty');
       await fillIn('[data-test-input="issuerName"]', 'foobar');
       assert.true(issuer.hasDirtyAttributes, 'Model is dirty');
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       issuers = this.store.peekAll('pki/issuer');
       assert.strictEqual(issuers.length, 1, 'Issuer exists on model in overview');
       issuer = this.store.peekRecord('pki/issuer', issuerId);
@@ -321,42 +330,42 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       await authPage.login();
       // Configure PKI -- key creation not allowed unless configured
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.emptyStateLink);
-      await click(SELECTORS.configuration.optionByKey('generate-root'));
-      await fillIn(SELECTORS.configuration.typeField, 'internal');
-      await fillIn(SELECTORS.configuration.inputByName('commonName'), 'my-root-cert');
-      await click(SELECTORS.configuration.generateRootSave);
+      await click(`${GENERAL.emptyStateActions} a`);
+      await click(PKI_CONFIGURE_CREATE.optionByKey('generate-root'));
+      await fillIn(GENERAL.inputByAttr('type'), 'internal');
+      await fillIn(GENERAL.inputByAttr('commonName'), 'my-root-cert');
+      await click(GENERAL.saveButton);
       await logout.visit();
     });
     test('create key exit', async function (assert) {
       let keys, key;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.keysTab);
+      await click(GENERAL.secretTab('Keys'));
       keys = this.store.peekAll('pki/key');
-      const configKeyId = keys.objectAt(0).id;
+      const configKeyId = keys.at(0).id;
       assert.strictEqual(keys.length, 1, 'One key exists from config');
       // Create key
-      await click(SELECTORS.keyPages.generateKey);
+      await click(PKI_KEYS.generateKey);
       keys = this.store.peekAll('pki/key');
-      key = keys.objectAt(1);
+      key = keys.at(1);
       assert.strictEqual(keys.length, 2, 'New key exists');
       assert.true(key.isNew, 'Role is new model');
       // Exit
-      await click(SELECTORS.keyForm.keyCancelButton);
+      await click(GENERAL.cancelButton);
       keys = this.store.peekAll('pki/key');
       assert.strictEqual(keys.length, 1, 'Second key is removed from store');
-      assert.strictEqual(keys.objectAt(0).id, configKeyId);
+      assert.strictEqual(keys.at(0).id, configKeyId);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/keys`, 'url is correct');
 
       // Create again
-      await click(SELECTORS.keyPages.generateKey);
+      await click(PKI_KEYS.generateKey);
       assert.strictEqual(keys.length, 2, 'New key exists');
       keys = this.store.peekAll('pki/key');
-      key = keys.objectAt(1);
+      key = keys.at(1);
       assert.true(key.isNew, 'Key is new model');
       // Exit
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/overview`, 'url is correct');
       keys = this.store.peekAll('pki/key');
       assert.strictEqual(keys.length, 1, 'Key is removed from store');
@@ -365,19 +374,19 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       let keys, key;
       await authPage.login();
       await visit(`/vault/secrets/${this.mountPath}/pki/overview`);
-      await click(SELECTORS.keysTab);
+      await click(GENERAL.secretTab('Keys'));
       keys = this.store.peekAll('pki/key');
       assert.strictEqual(keys.length, 1, 'One key from config exists');
       assert.dom('.list-item-row').exists({ count: 1 }, 'single row for key');
       await click('.list-item-row');
       // Edit
-      await click(SELECTORS.keyPages.keyEditLink);
-      await fillIn(SELECTORS.keyForm.keyNameInput, 'foobar');
+      await click(PKI_KEYS.keyEditLink);
+      await fillIn(GENERAL.inputByAttr('keyName'), 'foobar');
       keys = this.store.peekAll('pki/key');
-      key = keys.objectAt(0);
+      key = keys.at(0);
       assert.true(key.hasDirtyAttributes, 'Key model is dirty');
       // Exit
-      await click(SELECTORS.keyForm.keyCancelButton);
+      await click(GENERAL.cancelButton);
       assert.strictEqual(
         currentURL(),
         `/vault/secrets/${this.mountPath}/pki/keys/${key.id}/details`,
@@ -388,14 +397,14 @@ module('Acceptance | pki engine route cleanup test', function (hooks) {
       assert.false(key.hasDirtyAttributes, 'Key dirty attrs have been rolled back');
 
       // Edit again
-      await click(SELECTORS.keyPages.keyEditLink);
-      await fillIn(SELECTORS.keyForm.keyNameInput, 'foobar');
+      await click(PKI_KEYS.keyEditLink);
+      await fillIn(GENERAL.inputByAttr('keyName'), 'foobar');
       keys = this.store.peekAll('pki/key');
-      key = keys.objectAt(0);
+      key = keys.at(0);
       assert.true(key.hasDirtyAttributes, 'Key model is dirty');
 
       // Exit via breadcrumb
-      await click(SELECTORS.overviewBreadcrumb);
+      await click(OVERVIEW_BREADCRUMB);
       assert.strictEqual(currentURL(), `/vault/secrets/${this.mountPath}/pki/overview`, 'url is correct');
       keys = this.store.peekAll('pki/key');
       assert.strictEqual(keys.length, 1, 'Key list has 1');
