@@ -6,6 +6,7 @@
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
+import { v4 as uuidv4 } from 'uuid';
 import ldapMirageScenario from 'vault/mirage/scenarios/ldap';
 import ldapHandlers from 'vault/mirage/handlers/ldap';
 import authPage from 'vault/tests/pages/auth';
@@ -13,6 +14,7 @@ import { click, fillIn, waitFor } from '@ember/test-helpers';
 import { assertURL, isURL, visitURL } from 'vault/tests/helpers/ldap/ldap-helpers';
 import { GENERAL } from 'vault/tests/helpers/general-selectors';
 import { LDAP_SELECTORS } from 'vault/tests/helpers/ldap/ldap-selectors';
+import { deleteEngineCmd, mountEngineCmd, runCmd } from 'vault/tests/helpers/commands';
 
 module('Acceptance | ldap | roles', function (hooks) {
   setupApplicationTest(hooks);
@@ -21,14 +23,26 @@ module('Acceptance | ldap | roles', function (hooks) {
   hooks.beforeEach(async function () {
     ldapHandlers(this.server);
     ldapMirageScenario(this.server);
-    this.backend = 'ldap-test';
+    this.backend = `ldap-test-${uuidv4()}`;
     await authPage.login();
-    return visitURL('roles');
+    // mount & configure
+    await runCmd([
+      mountEngineCmd('ldap', this.backend),
+      `write ${this.backend}/config binddn=foo bindpass=bar url=http://localhost:8208`,
+    ]);
+    return visitURL('roles', this.backend);
+  });
+
+  hooks.afterEach(async function () {
+    await runCmd(deleteEngineCmd(this.backend));
   });
 
   test('it should transition to create role route on toolbar link click', async function (assert) {
     await click('[data-test-toolbar-action="role"]');
-    assert.true(isURL('roles/create'), 'Transitions to role create route on toolbar link click');
+    assert.true(
+      isURL('roles/create', this.backend),
+      'Transitions to role create route on toolbar link click'
+    );
   });
 
   test('it should transition to role details route on list item click', async function (assert) {
@@ -63,7 +77,7 @@ module('Acceptance | ldap | roles', function (hooks) {
       await click(LDAP_SELECTORS.action(action));
       const uri = action === 'get-creds' ? 'credentials' : action;
       assert.true(
-        isURL(`roles/dynamic/dynamic-role/${uri}`),
+        isURL(`roles/dynamic/dynamic-role/${uri}`, this.backend),
         `Transitions to ${uri} route on list item action menu click`
       );
       await click(GENERAL.breadcrumbLink('Roles'));
@@ -74,13 +88,16 @@ module('Acceptance | ldap | roles', function (hooks) {
     await click(LDAP_SELECTORS.roleItem('dynamic', 'dynamic-role'));
     await click('[data-test-get-credentials]');
     assert.true(
-      isURL('roles/dynamic/dynamic-role/credentials'),
+      isURL('roles/dynamic/dynamic-role/credentials', this.backend),
       'Transitions to credentials route from toolbar link'
     );
 
     await click('[data-test-breadcrumb="dynamic-role"] a');
     await click('[data-test-edit]');
-    assert.true(isURL('roles/dynamic/dynamic-role/edit'), 'Transitions to edit route from toolbar link');
+    assert.true(
+      isURL('roles/dynamic/dynamic-role/edit', this.backend),
+      'Transitions to edit route from toolbar link'
+    );
   });
 
   test('it should clear roles page filter value on route exit', async function (assert) {
@@ -88,6 +105,7 @@ module('Acceptance | ldap | roles', function (hooks) {
     assert
       .dom('[data-test-filter-input]')
       .hasValue('foo', 'Roles page filter value set after model refresh and rerender');
+    await waitFor(GENERAL.emptyStateTitle);
     await click('[data-test-tab="libraries"]');
     await click('[data-test-tab="roles"]');
     assert.dom('[data-test-filter-input]').hasNoValue('Roles page filter value cleared on route exit');
