@@ -3,10 +3,21 @@
  * SPDX-License-Identifier: BUSL-1.1
  */
 
-import { currentRouteName, currentURL, settled } from '@ember/test-helpers';
+import {
+  currentRouteName,
+  currentURL,
+  settled,
+  click,
+  findAll,
+  fillIn,
+  visit,
+  typeIn,
+} from '@ember/test-helpers';
+import { clickTrigger } from 'ember-power-select/test-support/helpers';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { v4 as uuidv4 } from 'uuid';
+import { runCmd, tokenWithPolicyCmd } from 'vault/tests/helpers/commands';
 
 import { create } from 'ember-cli-page-object';
 import page from 'vault/tests/pages/settings/mount-secret-backend';
@@ -17,6 +28,11 @@ import logout from 'vault/tests/pages/logout';
 import mountSecrets from 'vault/tests/pages/settings/mount-secret-backend';
 import { mountableEngines } from 'vault/helpers/mountable-secret-engines'; // allEngines() includes enterprise engines, those are tested elsewhere
 import { supportedSecretBackends } from 'vault/helpers/supported-secret-backends';
+import { GENERAL } from 'vault/tests/helpers/general-selectors';
+import { SECRET_ENGINE_SELECTORS as SES } from 'vault/tests/helpers/secret-engine/secret-engine-selectors';
+import { SELECTORS as OIDC } from 'vault/tests/helpers/oidc-config';
+import { adminOidcCreateRead, adminOidcCreate } from 'vault/tests/helpers/secret-engine/policy-generator';
+import { WIF_ENGINES } from 'vault/helpers/mountable-secret-engines';
 
 const consoleComponent = create(consoleClass);
 
@@ -45,7 +61,6 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     assert.strictEqual(currentRouteName(), 'vault.cluster.settings.mount-secret-backend');
     await page.selectType('kv');
     await page
-      .next()
       .path(path)
       .toggleOptions()
       .enableDefaultTtl()
@@ -70,7 +85,6 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     assert.strictEqual(currentRouteName(), 'vault.cluster.settings.mount-secret-backend');
     await page.selectType('kv');
     await page
-      .next()
       .path(path)
       .toggleOptions()
       .enableDefaultTtl()
@@ -79,11 +93,7 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
       .maxTTLVal(maxTTLHours)
       .submit();
     await configPage.visit({ backend: path });
-    assert.strictEqual(
-      configPage.defaultTTL,
-      '0',
-      'shows 0 (with no seconds) which means using the system default TTL'
-    ); // https://developer.hashicorp.com/vault/api-docs/system/mounts#default_lease_ttl-1
+    assert.strictEqual(configPage.defaultTTL, '1 month 1 day', 'shows system default TTL');
     assert.strictEqual(configPage.maxTTL, `${this.calcDays(maxTTLHours)}`, 'shows the proper max TTL');
   });
 
@@ -91,7 +101,6 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     await page.visit();
     assert.strictEqual(currentRouteName(), 'vault.cluster.settings.mount-secret-backend');
     await page.selectType('pki');
-    await page.next();
     assert.dom('[data-test-input="maxLeaseTtl"]').exists();
     assert
       .dom('[data-test-input="maxLeaseTtl"] [data-test-ttl-toggle]')
@@ -102,7 +111,6 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     // Go back and choose a different type
     await page.back();
     await page.selectType('database');
-    await page.next();
     assert.dom('[data-test-input="maxLeaseTtl"]').exists('3650');
     assert
       .dom('[data-test-input="maxLeaseTtl"] [data-test-ttl-toggle]')
@@ -124,12 +132,12 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
 
     assert.strictEqual(currentRouteName(), 'vault.cluster.settings.mount-secret-backend');
     await page.selectType('kv');
-    await page.next().path(path).submit();
+    await page.path(path).submit();
     await page.secretList();
     await settled();
     await page.enableEngine();
     await page.selectType('kv');
-    await page.next().path(path).submit();
+    await page.path(path).submit();
     assert.dom('[data-test-message-error-description]').containsText(`path is already in use at ${path}`);
     assert.strictEqual(currentRouteName(), 'vault.cluster.settings.mount-secret-backend');
 
@@ -161,12 +169,16 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
         capabilities = ["read"]
       }
     `;
-    await consoleComponent.runCommands([
-      // delete any previous mount with same name
-      `delete sys/mounts/${enginePath}`,
-      `write sys/policies/acl/kv-v2-degrade policy=${btoa(V2_POLICY)}`,
-      'write -field=client_token auth/token/create policies=kv-v2-degrade',
-    ]);
+    await consoleComponent.toggle();
+    await consoleComponent.runCommands(
+      [
+        // delete any previous mount with same name
+        `delete sys/mounts/${enginePath}`,
+        `write sys/policies/acl/kv-v2-degrade policy=${btoa(V2_POLICY)}`,
+        'write -field=client_token auth/token/create policies=kv-v2-degrade',
+      ],
+      false
+    );
     await settled();
     const userToken = consoleComponent.lastLogOutput;
     await logout.visit();
@@ -174,7 +186,7 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     // create the engine
     await mountSecrets.visit();
     await mountSecrets.selectType('kv');
-    await mountSecrets.next().path(enginePath).setMaxVersion(101).submit();
+    await mountSecrets.path(enginePath).setMaxVersion(101).submit();
     await settled();
     assert
       .dom('[data-test-flash-message]')
@@ -188,7 +200,6 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     );
     await configPage.visit({ backend: enginePath });
     await settled();
-    assert.dom('[data-test-row-value="Maximum number of versions"]').hasText('Not set');
   });
 
   test('it should transition to mountable addon engine after mount success', async function (assert) {
@@ -203,7 +214,7 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
       ]);
       await mountSecrets.visit();
       await mountSecrets.selectType(engine.type);
-      await mountSecrets.next().path(engine.type).submit();
+      await mountSecrets.path(engine.type).submit();
       assert.strictEqual(
         currentRouteName(),
         `vault.cluster.secrets.backend.${engine.engineRoute}`,
@@ -230,7 +241,7 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
       ]);
       await mountSecrets.visit();
       await mountSecrets.selectType(engine.type);
-      await mountSecrets.next().path(engine.type);
+      await mountSecrets.path(engine.type);
       if (engine.type === 'kv') {
         await mountSecrets.toggleOptions().version(1);
       }
@@ -259,7 +270,7 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
       ]);
       await mountSecrets.visit();
       await mountSecrets.selectType(engine.type);
-      await mountSecrets.next().path(engine.type).submit();
+      await mountSecrets.path(engine.type).submit();
 
       assert.strictEqual(
         currentRouteName(),
@@ -278,7 +289,7 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     ]);
     await mountSecrets.visit();
     await mountSecrets.selectType('kv');
-    await mountSecrets.next().path(v2).submit();
+    await mountSecrets.path(v2).submit();
 
     assert.strictEqual(currentURL(), `/vault/secrets/${v2}/kv/list`, `${v2} navigates to list url`);
     assert.strictEqual(
@@ -294,7 +305,7 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
     ]);
     await mountSecrets.visit();
     await mountSecrets.selectType('kv');
-    await mountSecrets.next().path(v1).toggleOptions().version(1).submit();
+    await mountSecrets.path(v1).toggleOptions().version(1).submit();
 
     assert.strictEqual(currentURL(), `/vault/secrets/${v1}/list`, `${v1} navigates to list url`);
     assert.strictEqual(
@@ -303,4 +314,107 @@ module('Acceptance | settings/mount-secret-backend', function (hooks) {
       `${v1} navigates to list route`
     );
   });
+
+  module('WIF secret engines', function () {
+    test('it sets identity_token_key on mount config using search select list, resets after', async function (assert) {
+      // create an oidc/key
+      await runCmd(`write identity/oidc/key/some-key allowed_client_ids="*"`);
+
+      for (const engine of WIF_ENGINES) {
+        await page.visit();
+        await page.selectType(engine);
+        await click(GENERAL.toggleGroup('Method Options'));
+        assert
+          .dom('[data-test-search-select-with-modal]')
+          .exists('Search select with modal component renders');
+        await clickTrigger('#key');
+        const dropdownOptions = findAll('[data-option-index]').map((o) => o.innerText);
+        assert.ok(dropdownOptions.includes('some-key'), 'search select options show some-key');
+        await click(GENERAL.searchSelect.option(GENERAL.searchSelect.optionIndex('some-key')));
+        assert
+          .dom(GENERAL.searchSelect.selectedOption())
+          .hasText('some-key', 'some-key was selected and displays in the search select');
+      }
+      // Go back and choose a non-wif engine type
+      await page.back();
+      await page.selectType('ssh');
+      assert
+        .dom('[data-test-search-select-with-modal]')
+        .doesNotExist('for type ssh, the modal field does not render.');
+      // cleanup
+      await runCmd(`delete identity/oidc/key/some-key`);
+    });
+
+    test('it allows a user with permissions to oidc/key to create an identity_token_key', async function (assert) {
+      for (const engine of WIF_ENGINES) {
+        const path = `secrets-adminPolicy-${engine}`;
+        const newKey = `key-${uuidv4()}`;
+        const secrets_admin_policy = adminOidcCreateRead(path);
+        const secretsAdminToken = await runCmd(
+          tokenWithPolicyCmd(`secrets-admin-${path}`, secrets_admin_policy)
+        );
+
+        await logout.visit();
+        await authPage.login(secretsAdminToken);
+        await page.visit();
+        await page.selectType(engine);
+        await page.path(path);
+        await click(GENERAL.toggleGroup('Method Options'));
+        await clickTrigger('#key');
+        // create new key
+        await fillIn(GENERAL.searchSelect.searchInput, newKey);
+        await click(GENERAL.searchSelect.options);
+        assert.dom('#search-select-modal').exists('modal with form opens');
+        assert.dom('[data-test-modal-title]').hasText('Create new key', 'Create key modal renders');
+
+        await click(OIDC.keySaveButton);
+        assert.dom('#search-select-modal').doesNotExist('modal disappears onSave');
+        assert.dom(GENERAL.searchSelect.selectedOption()).hasText(newKey, `${newKey} is now selected`);
+
+        await page.submit();
+        await visit(`/vault/secrets/${path}/configuration`);
+        await click(SES.configurationToggle);
+        assert
+          .dom(GENERAL.infoRowValue('Identity Token Key'))
+          .hasText(newKey, 'shows identity token key on configuration page');
+        // cleanup
+        await runCmd(`delete sys/mounts/${path}`);
+        await runCmd(`delete identity/oidc/key/some-key`);
+        await runCmd(`delete identity/oidc/key/${newKey}`);
+      }
+    });
+
+    test('it allows user with NO access to oidc/key to manually input an identity_token_key', async function (assert) {
+      for (const engine of WIF_ENGINES) {
+        const path = `secrets-noOidcAdmin-${engine}`;
+        const secretsNoOidcAdminPolicy = adminOidcCreate(path);
+        const secretsNoOidcAdminToken = await runCmd(
+          tokenWithPolicyCmd(`secrets-noOidcAdmin-${path}`, secretsNoOidcAdminPolicy)
+        );
+        // create an oidc/key that they can then use even if they can't read it.
+        await runCmd(`write identity/oidc/key/general-key allowed_client_ids="*"`);
+
+        await logout.visit();
+        await authPage.login(secretsNoOidcAdminToken);
+        await page.visit();
+        await page.selectType(engine);
+        await page.path(path);
+        await click(GENERAL.toggleGroup('Method Options'));
+        // type-in fallback component to create new key
+        await typeIn(GENERAL.inputSearch('key'), 'general-key');
+        await page.submit();
+        assert
+          .dom(GENERAL.latestFlashContent)
+          .hasText(`Successfully mounted the aws secrets engine at ${path}.`);
+
+        await visit(`/vault/secrets/${path}/configuration`);
+        await click(SES.configurationToggle);
+        assert
+          .dom(GENERAL.infoRowValue('Identity Token Key'))
+          .hasText('general-key', 'shows identity token key on configuration page');
+        // cleanup
+        await runCmd(`delete sys/mounts/${path}`);
+      }
+    });
+  });
 });
