@@ -6,16 +6,23 @@
 import { allFeatures } from 'vault/helpers/all-features';
 import sinon from 'sinon';
 
-export const stubFeaturesAndPermissions = (owner, isEnterprise = false, setCluster = false) => {
+/**
+ * Sets up the necessary shared test context for testing sidebar nav components
+ * @param {TestContext.owner} owner eg this.owner from within a test
+ * @param {boolean} isEnterprise [default false]
+ * @param {boolean} setCluster [default false] if true, will set the current cluster to one with replication & raft
+ * @param {string[]} features if not passed, defaults to all features
+ * @returns {hasNavPermission: sinon.SinonStub, features: string[]}
+ */
+export const stubFeaturesAndPermissions = (owner, isEnterprise = false, setCluster = false, features) => {
   const permissions = owner.lookup('service:permissions');
   const hasNavPermission = sinon.stub(permissions, 'hasNavPermission');
   hasNavPermission.returns(true);
   sinon.stub(permissions, 'navPathParams');
 
   const version = owner.lookup('service:version');
-  const features = sinon.stub(version, 'features');
-  features.value(allFeatures());
-  sinon.stub(version, 'isEnterprise').value(isEnterprise);
+  version.type = isEnterprise ? 'enterprise' : 'community';
+  version.features = features || allFeatures();
 
   const auth = owner.lookup('service:auth');
   sinon.stub(auth, 'authData').value({});
