@@ -7,6 +7,6 @@ import { settled } from '@ember/test-helpers';
 
 export async function pollCluster(owner) {
   const store = owner.lookup('service:store');
-  await store.peekAll('cluster').firstObject.reload();
+  await store.peekAll('cluster')[0].reload();
   await settled();
 }
