@@ -1,18 +1,23 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import { module, test } from 'qunit';
 import { setupRenderingTest } from 'ember-qunit';
 import { render } from '@ember/test-helpers';
 import hbs from 'htmlbars-inline-precompile';
 
-module('Integration | Component | console/log text', function(hooks) {
+module('Integration | Component | console/log text', function (hooks) {
   setupRenderingTest(hooks);
 
-  test('it renders', async function(assert) {
+  test('it renders', async function (assert) {
     // Set any properties with this.set('myProperty', 'value');
     // Handle any actions with this.on('myAction', function(val) { ... });
     const text = 'Success! You did a thing!';
     this.set('content', text);
 
-    await render(hbs`{{console/log-text content=content}}`);
+    await render(hbs`{{console/log-text content=this.content}}`);
 
     assert.dom('pre').includesText(text);
   });
