@@ -10,9 +10,9 @@ import { setupEngine } from 'ember-engines/test-support';
 import { hbs } from 'ember-cli-htmlbars';
 import { setupMirage } from 'ember-cli-mirage/test-support';
 import { Response } from 'miragejs';
-import { SELECTORS } from 'vault/tests/helpers/pki/page/pki-configuration-edit';
 import sinon from 'sinon';
 import { allowAllCapabilitiesStub } from 'vault/tests/helpers/stubs';
+import { PKI_CONFIG_EDIT } from 'vault/tests/helpers/pki/pki-selectors';
 
 module('Integration | Component | page/pki-configuration-edit', function (hooks) {
   setupRenderingTest(hooks);
@@ -137,70 +137,70 @@ module('Integration | Component | page/pki-configuration-edit', function (hooks)
       this.context
     );
 
-    assert.dom(SELECTORS.configEditSection).exists('renders config section');
-    assert.dom(SELECTORS.urlsEditSection).exists('renders urls section');
-    assert.dom(SELECTORS.crlEditSection).exists('renders crl section');
-    assert.dom(SELECTORS.cancelButton).exists();
+    assert.dom(PKI_CONFIG_EDIT.configEditSection).exists('renders config section');
+    assert.dom(PKI_CONFIG_EDIT.urlsEditSection).exists('renders urls section');
+    assert.dom(PKI_CONFIG_EDIT.crlEditSection).exists('renders crl section');
+    assert.dom(PKI_CONFIG_EDIT.cancelButton).exists();
     this.urls.eachAttribute((name) => {
-      assert.dom(SELECTORS.urlFieldInput(name)).exists(`renders ${name} input`);
+      assert.dom(PKI_CONFIG_EDIT.urlFieldInput(name)).exists(`renders ${name} input`);
     });
-    assert.dom(SELECTORS.urlFieldInput('issuingCertificates')).hasValue('hashicorp.com');
-    assert.dom(SELECTORS.urlFieldInput('crlDistributionPoints')).hasValue('some-crl-distribution.com');
-    assert.dom(SELECTORS.urlFieldInput('ocspServers')).hasValue('ocsp-stuff.com');
+    assert.dom(PKI_CONFIG_EDIT.urlFieldInput('issuingCertificates')).hasValue('hashicorp.com');
+    assert.dom(PKI_CONFIG_EDIT.urlFieldInput('crlDistributionPoints')).hasValue('some-crl-distribution.com');
+    assert.dom(PKI_CONFIG_EDIT.urlFieldInput('ocspServers')).hasValue('ocsp-stuff.com');
 
     // cluster config
-    await fillIn(SELECTORS.configInput('path'), 'https://pr-a.vault.example.com/v1/ns1/pki-root');
-    await fillIn(SELECTORS.configInput('aiaPath'), 'http://another-path.com');
+    await fillIn(PKI_CONFIG_EDIT.configInput('path'), 'https://pr-a.vault.example.com/v1/ns1/pki-root');
+    await fillIn(PKI_CONFIG_EDIT.configInput('aiaPath'), 'http://another-path.com');
 
     // acme config;
-    await click(SELECTORS.configInput('enabled'));
-    await fillIn(SELECTORS.stringListInput('allowedRoles'), 'my-role');
-    await fillIn(SELECTORS.stringListInput('allowedIssuers'), '*');
-    await fillIn(SELECTORS.configInput('eabPolicy'), 'new-account-required');
-    await fillIn(SELECTORS.configInput('dnsResolver'), 'some-dns');
+    await click(PKI_CONFIG_EDIT.configInput('enabled'));
+    await fillIn(PKI_CONFIG_EDIT.stringListInput('allowedRoles'), 'my-role');
+    await fillIn(PKI_CONFIG_EDIT.stringListInput('allowedIssuers'), '*');
+    await fillIn(PKI_CONFIG_EDIT.configInput('eabPolicy'), 'new-account-required');
+    await fillIn(PKI_CONFIG_EDIT.configInput('dnsResolver'), 'some-dns');
 
     // urls
-    await fillIn(SELECTORS.urlFieldInput('issuingCertificates'), 'update-hashicorp.com');
-    await fillIn(SELECTORS.urlFieldInput('crlDistributionPoints'), 'test-crl.com');
-    await fillIn(SELECTORS.urlFieldInput('ocspServers'), 'ocsp.com');
+    await fillIn(PKI_CONFIG_EDIT.urlFieldInput('issuingCertificates'), 'update-hashicorp.com');
+    await fillIn(PKI_CONFIG_EDIT.urlFieldInput('crlDistributionPoints'), 'test-crl.com');
+    await fillIn(PKI_CONFIG_EDIT.urlFieldInput('ocspServers'), 'ocsp.com');
 
     // confirm default toggle state and text
     this.crl.eachAttribute((name, { options }) => {
       if (['expiry', 'ocspExpiry'].includes(name)) {
-        assert.dom(SELECTORS.crlToggleInput(name)).isChecked(`${name} defaults to toggled on`);
-        assert.dom(SELECTORS.crlFieldLabel(name)).hasTextContaining(options.label);
-        assert.dom(SELECTORS.crlFieldLabel(name)).hasTextContaining(options.helperTextEnabled);
+        assert.dom(PKI_CONFIG_EDIT.crlToggleInput(name)).isChecked(`${name} defaults to toggled on`);
+        assert.dom(PKI_CONFIG_EDIT.crlFieldLabel(name)).hasTextContaining(options.label);
+        assert.dom(PKI_CONFIG_EDIT.crlFieldLabel(name)).hasTextContaining(options.helperTextEnabled);
       }
       if (['autoRebuildGracePeriod', 'deltaRebuildInterval'].includes(name)) {
-        assert.dom(SELECTORS.crlToggleInput(name)).isNotChecked(`${name} defaults off`);
-        assert.dom(SELECTORS.crlFieldLabel(name)).hasTextContaining(options.labelDisabled);
-        assert.dom(SELECTORS.crlFieldLabel(name)).hasTextContaining(options.helperTextDisabled);
+        assert.dom(PKI_CONFIG_EDIT.crlToggleInput(name)).isNotChecked(`${name} defaults off`);
+        assert.dom(PKI_CONFIG_EDIT.crlFieldLabel(name)).hasTextContaining(options.labelDisabled);
+        assert.dom(PKI_CONFIG_EDIT.crlFieldLabel(name)).hasTextContaining(options.helperTextDisabled);
       }
     });
 
     // toggle everything on
-    await click(SELECTORS.crlToggleInput('autoRebuildGracePeriod'));
+    await click(PKI_CONFIG_EDIT.crlToggleInput('autoRebuildGracePeriod'));
     assert
-      .dom(SELECTORS.crlFieldLabel('autoRebuildGracePeriod'))
+      .dom(PKI_CONFIG_EDIT.crlFieldLabel('autoRebuildGracePeriod'))
       .hasTextContaining(
         'Auto-rebuild on Vault will rebuild the CRL in the below grace period before expiration',
         'it renders auto rebuild toggled on text'
       );
-    await click(SELECTORS.crlToggleInput('deltaRebuildInterval'));
+    await click(PKI_CONFIG_EDIT.crlToggleInput('deltaRebuildInterval'));
     assert
-      .dom(SELECTORS.crlFieldLabel('deltaRebuildInterval'))
+      .dom(PKI_CONFIG_EDIT.crlFieldLabel('deltaRebuildInterval'))
       .hasTextContaining(
         'Delta CRL building on Vault will rebuild the delta CRL at the interval below:',
         'it renders delta crl build toggled on text'
       );
 
     // assert ttl values update model attributes
-    await fillIn(SELECTORS.crlTtlInput('Expiry'), '48');
-    await fillIn(SELECTORS.crlTtlInput('Auto-rebuild on'), '24');
-    await fillIn(SELECTORS.crlTtlInput('Delta CRL building on'), '45');
-    await fillIn(SELECTORS.crlTtlInput('OCSP responder APIs enabled'), '24');
+    await fillIn(PKI_CONFIG_EDIT.crlTtlInput('Expiry'), '48');
+    await fillIn(PKI_CONFIG_EDIT.crlTtlInput('Auto-rebuild on'), '24');
+    await fillIn(PKI_CONFIG_EDIT.crlTtlInput('Delta CRL building on'), '45');
+    await fillIn(PKI_CONFIG_EDIT.crlTtlInput('OCSP responder APIs enabled'), '24');
 
-    await click(SELECTORS.saveButton);
+    await click(PKI_CONFIG_EDIT.saveButton);
   });
 
   test('it removes urls and sends false crl values', async function (assert) {
@@ -249,34 +249,34 @@ module('Integration | Component | page/pki-configuration-edit', function (hooks)
       this.context
     );
 
-    await click(SELECTORS.deleteButton('issuingCertificates'));
-    await click(SELECTORS.deleteButton('crlDistributionPoints'));
-    await click(SELECTORS.deleteButton('ocspServers'));
+    await click(PKI_CONFIG_EDIT.deleteButton('issuingCertificates'));
+    await click(PKI_CONFIG_EDIT.deleteButton('crlDistributionPoints'));
+    await click(PKI_CONFIG_EDIT.deleteButton('ocspServers'));
 
     // toggle everything off
-    await click(SELECTORS.crlToggleInput('expiry'));
-    assert.dom(SELECTORS.crlFieldLabel('expiry')).hasText('No expiry The CRL will not be built.');
+    await click(PKI_CONFIG_EDIT.crlToggleInput('expiry'));
+    assert.dom(PKI_CONFIG_EDIT.crlFieldLabel('expiry')).hasText('No expiry The CRL will not be built.');
     assert
-      .dom(SELECTORS.crlToggleInput('autoRebuildGracePeriod'))
+      .dom(PKI_CONFIG_EDIT.crlToggleInput('autoRebuildGracePeriod'))
       .doesNotExist('expiry off hides the auto rebuild toggle');
     assert
-      .dom(SELECTORS.crlToggleInput('deltaRebuildInterval'))
+      .dom(PKI_CONFIG_EDIT.crlToggleInput('deltaRebuildInterval'))
       .doesNotExist('expiry off hides delta crl toggle');
-    await click(SELECTORS.crlToggleInput('ocspExpiry'));
+    await click(PKI_CONFIG_EDIT.crlToggleInput('ocspExpiry'));
     assert
-      .dom(SELECTORS.crlFieldLabel('ocspExpiry'))
+      .dom(PKI_CONFIG_EDIT.crlFieldLabel('ocspExpiry'))
       .hasTextContaining(
         'OCSP responder APIs disabled Requests cannot be made to check if an individual certificate is valid.',
         'it renders correct toggled off text'
       );
 
-    await click(SELECTORS.saveButton);
+    await click(PKI_CONFIG_EDIT.saveButton);
   });
 
   test('it renders enterprise only params', async function (assert) {
     assert.expect(6);
     this.version = this.owner.lookup('service:version');
-    this.version.version = '1.13.1+ent';
+    this.version.type = 'enterprise';
     this.server.post(`/${this.backend}/config/acme`, () => {});
     this.server.post(`/${this.backend}/config/cluster`, () => {});
     this.server.post(`/${this.backend}/config/crl`, (schema, req) => {
@@ -315,19 +315,19 @@ module('Integration | Component | page/pki-configuration-edit', function (hooks)
       this.context
     );
 
-    assert.dom(SELECTORS.groupHeader('Certificate Revocation List (CRL)')).exists();
-    assert.dom(SELECTORS.groupHeader('Online Certificate Status Protocol (OCSP)')).exists();
-    assert.dom(SELECTORS.groupHeader('Unified Revocation')).exists();
-    await click(SELECTORS.checkboxInput('crossClusterRevocation'));
-    await click(SELECTORS.checkboxInput('unifiedCrl'));
-    await click(SELECTORS.checkboxInput('unifiedCrlOnExistingPaths'));
-    await click(SELECTORS.saveButton);
+    assert.dom(PKI_CONFIG_EDIT.groupHeader('Certificate Revocation List (CRL)')).exists();
+    assert.dom(PKI_CONFIG_EDIT.groupHeader('Online Certificate Status Protocol (OCSP)')).exists();
+    assert.dom(PKI_CONFIG_EDIT.groupHeader('Unified Revocation')).exists();
+    await click(PKI_CONFIG_EDIT.checkboxInput('crossClusterRevocation'));
+    await click(PKI_CONFIG_EDIT.checkboxInput('unifiedCrl'));
+    await click(PKI_CONFIG_EDIT.checkboxInput('unifiedCrlOnExistingPaths'));
+    await click(PKI_CONFIG_EDIT.saveButton);
   });
 
   test('it does not render enterprise only params for OSS', async function (assert) {
     assert.expect(9);
     this.version = this.owner.lookup('service:version');
-    this.version.version = '1.13.1';
+    this.version.type = 'community';
     this.server.post(`/${this.backend}/config/acme`, () => {});
     this.server.post(`/${this.backend}/config/cluster`, () => {});
     this.server.post(`/${this.backend}/config/crl`, (schema, req) => {
@@ -363,13 +363,13 @@ module('Integration | Component | page/pki-configuration-edit', function (hooks)
       this.context
     );
 
-    assert.dom(SELECTORS.checkboxInput('crossClusterRevocation')).doesNotExist();
-    assert.dom(SELECTORS.checkboxInput('unifiedCrl')).doesNotExist();
-    assert.dom(SELECTORS.checkboxInput('unifiedCrlOnExistingPaths')).doesNotExist();
-    assert.dom(SELECTORS.groupHeader('Certificate Revocation List (CRL)')).exists();
-    assert.dom(SELECTORS.groupHeader('Online Certificate Status Protocol (OCSP)')).exists();
-    assert.dom(SELECTORS.groupHeader('Unified Revocation')).doesNotExist();
-    await click(SELECTORS.saveButton);
+    assert.dom(PKI_CONFIG_EDIT.checkboxInput('crossClusterRevocation')).doesNotExist();
+    assert.dom(PKI_CONFIG_EDIT.checkboxInput('unifiedCrl')).doesNotExist();
+    assert.dom(PKI_CONFIG_EDIT.checkboxInput('unifiedCrlOnExistingPaths')).doesNotExist();
+    assert.dom(PKI_CONFIG_EDIT.groupHeader('Certificate Revocation List (CRL)')).exists();
+    assert.dom(PKI_CONFIG_EDIT.groupHeader('Online Certificate Status Protocol (OCSP)')).exists();
+    assert.dom(PKI_CONFIG_EDIT.groupHeader('Unified Revocation')).doesNotExist();
+    await click(PKI_CONFIG_EDIT.saveButton);
   });
 
   test('it renders empty states if no update capabilities', async function (assert) {
@@ -390,22 +390,22 @@ module('Integration | Component | page/pki-configuration-edit', function (hooks)
     );
 
     assert
-      .dom(`${SELECTORS.configEditSection} [data-test-component="empty-state"]`)
+      .dom(`${PKI_CONFIG_EDIT.configEditSection} [data-test-component="empty-state"]`)
       .hasText(
         "You do not have permission to set this mount's the cluster config Ask your administrator if you think you should have access to: POST /pki-engine/config/cluster"
       );
     assert
-      .dom(`${SELECTORS.acmeEditSection} [data-test-component="empty-state"]`)
+      .dom(`${PKI_CONFIG_EDIT.acmeEditSection} [data-test-component="empty-state"]`)
       .hasText(
         "You do not have permission to set this mount's ACME config Ask your administrator if you think you should have access to: POST /pki-engine/config/acme"
       );
     assert
-      .dom(`${SELECTORS.urlsEditSection} [data-test-component="empty-state"]`)
+      .dom(`${PKI_CONFIG_EDIT.urlsEditSection} [data-test-component="empty-state"]`)
       .hasText(
         "You do not have permission to set this mount's URLs Ask your administrator if you think you should have access to: POST /pki-engine/config/urls"
       );
     assert
-      .dom(`${SELECTORS.crlEditSection} [data-test-component="empty-state"]`)
+      .dom(`${PKI_CONFIG_EDIT.crlEditSection} [data-test-component="empty-state"]`)
       .hasText(
         "You do not have permission to set this mount's revocation configuration Ask your administrator if you think you should have access to: POST /pki-engine/config/crl"
       );
@@ -438,21 +438,21 @@ module('Integration | Component | page/pki-configuration-edit', function (hooks)
       this.context
     );
 
-    await click(SELECTORS.saveButton);
+    await click(PKI_CONFIG_EDIT.saveButton);
     assert
-      .dom(SELECTORS.errorBanner)
+      .dom(PKI_CONFIG_EDIT.errorBanner)
       .hasText(
         'Error POST config/cluster: something wrong with cluster POST config/acme: something wrong with acme POST config/urls: something wrong with urls POST config/crl: something wrong with crl'
       );
-    assert.dom(`${SELECTORS.errorBanner} ul`).hasClass('bullet');
+    assert.dom(`${PKI_CONFIG_EDIT.errorBanner} ul`).hasClass('bullet');
 
     // change 3 out of 4 requests to be successful to assert single error renders correctly
     this.server.post(`/${this.backend}/config/acme`, () => new Response(200));
     this.server.post(`/${this.backend}/config/cluster`, () => new Response(200));
     this.server.post(`/${this.backend}/config/crl`, () => new Response(200));
 
-    await click(SELECTORS.saveButton);
-    assert.dom(SELECTORS.errorBanner).hasText('Error POST config/urls: something wrong with urls');
-    assert.dom(`${SELECTORS.errorBanner} ul`).doesNotHaveClass('bullet');
+    await click(PKI_CONFIG_EDIT.saveButton);
+    assert.dom(PKI_CONFIG_EDIT.errorBanner).hasText('Error POST config/urls: something wrong with urls');
+    assert.dom(`${PKI_CONFIG_EDIT.errorBanner} ul`).doesNotHaveClass('bullet');
   });
 });
