@@ -7,6 +7,7 @@ import { module, test } from 'qunit';
 import { setupRenderingTest } from 'ember-qunit';
 import { render } from '@ember/test-helpers';
 import hbs from 'htmlbars-inline-precompile';
+import { setRunOptions } from 'ember-a11y-testing/test-support';
 
 const TITLE = 'Status';
 
@@ -28,6 +29,13 @@ module('Integration | Component | replication-secondary-card', function (hooks)
   hooks.beforeEach(function () {
     this.set('replicationDetails', REPLICATION_DETAILS);
     this.set('title', TITLE);
+    // TODO: remove Tooltip/ember-basic-dropdown
+    setRunOptions({
+      rules: {
+        'aria-command-name': { enabled: false },
+        'nested-interactive': { enabled: false },
+      },
+    });
   });
 
   test('it renders', async function (assert) {
@@ -48,10 +56,10 @@ module('Integration | Component | replication-secondary-card', function (hooks)
 
     assert.dom('[data-test-info-table]').exists('it shows the known primary cluster details');
 
-    const url = this.element.querySelector('[data-test-primary-link]').href;
     const expectedUrl = `${REPLICATION_DETAILS.primaries[0].api_address}/ui/`;
-
-    assert.strictEqual(url, expectedUrl, 'it renders a link to the primary cluster UI');
+    assert
+      .dom('[data-test-primary-link]')
+      .hasAttribute('href', expectedUrl, 'it renders a link to the primary cluster UI');
   });
 
   test('it does not render a link to the primary cluster UI when the primary api address or known primaries are unknown', async function (assert) {
@@ -68,14 +76,14 @@ module('Integration | Component | replication-secondary-card', function (hooks)
     await render(
       hbs`<ReplicationSecondaryCard @replicationDetails={{this.replicationDetails}} @title='Primary cluster'/>`
     );
-    assert.dom('[data-test-component="empty-state"]').exists();
+    assert.dom('[data-test-empty-state]').exists('shows empty state');
   });
 
-  test('it renders tooltip with check-circle-outline when state is stream-wals', async function (assert) {
+  test('it renders tooltip with check-circle when state is stream-wals', async function (assert) {
     await render(
       hbs`<ReplicationSecondaryCard @replicationDetails={{this.replicationDetails}} @title={{this.title}} />`
     );
-    assert.dom('[data-test-glyph]').hasClass('has-text-success', `shows success icon`);
+    assert.dom('[data-test-icon="check-circle"]').hasClass('has-text-success', `shows success icon`);
   });
 
   test('it renders hasErrorMessage when state is idle', async function (assert) {
