@@ -10,6 +10,7 @@ import { resolve } from 'rsvp';
 import { run } from '@ember/runloop';
 import Service from '@ember/service';
 import hbs from 'htmlbars-inline-precompile';
+import codemirror from 'vault/tests/helpers/codemirror';
 
 let capabilities;
 const storeService = Service.extend({
@@ -22,7 +23,7 @@ module('Integration | Component | secret edit', function (hooks) {
 
   hooks.beforeEach(function () {
     capabilities = null;
-    this.codeMirror = this.owner.lookup('service:code-mirror');
+    this.set('key', { id: 'Foobar' });
     run(() => {
       this.owner.unregister('service:store');
       this.owner.register('service:store', storeService);
@@ -39,7 +40,7 @@ module('Integration | Component | secret edit', function (hooks) {
       },
     });
 
-    await render(hbs`{{secret-edit mode=this.mode model=this.model }}`);
+    await render(hbs`<SecretEdit @mode={{this.mode}} @model={{this.model}} @key={{this.key}} />`);
     assert.dom('[data-test-toggle-input="json"]').isDisabled();
   });
 
@@ -53,7 +54,7 @@ module('Integration | Component | secret edit', function (hooks) {
       },
     });
 
-    await render(hbs`{{secret-edit mode=this.mode model=this.model }}`);
+    await render(hbs`<SecretEdit @mode={{this.mode}} @model={{this.model}} @key={{this.key}} />`);
     assert.dom('[data-test-toggle-input="json"]').isNotDisabled();
   });
 
@@ -63,10 +64,11 @@ module('Integration | Component | secret edit', function (hooks) {
       secretData: null,
     });
 
-    await render(hbs`{{secret-edit mode=this.mode model=this.model preferAdvancedEdit=true }}`);
+    await render(
+      hbs`<SecretEdit @mode={{this.mode}} @model={{this.model}} @preferAdvancedEdit={{true}} @key={{this.key}} />`
+    );
 
-    const instance = document.querySelector('.CodeMirror').CodeMirror;
-    instance.setValue(JSON.stringify([{ foo: 'bar' }]));
+    codemirror().setValue(JSON.stringify([{ foo: 'bar' }]));
     await settled();
     assert
       .dom('[data-test-message-error]')
@@ -83,7 +85,7 @@ module('Integration | Component | secret edit', function (hooks) {
         float: '1.234',
       },
     });
-    await render(hbs`<SecretEdit @mode={{this.mode}} @model={{this.model}} />`);
+    await render(hbs`<SecretEdit @mode={{this.mode}} @model={{this.model}} @key={{this.key}} />`);
     assert.dom('[data-test-secret-save]').isNotDisabled();
   });
 
@@ -101,10 +103,11 @@ module('Integration | Component | secret edit', function (hooks) {
       canReadSecretData: true,
     });
 
-    await render(hbs`{{secret-edit mode=this.mode model=this.model preferAdvancedEdit=true }}`);
+    await render(
+      hbs`<SecretEdit @mode={{this.mode}} @model={{this.model}} @preferAdvancedEdit={{true}} @key={{this.key}} />`
+    );
 
-    const instance = document.querySelector('.CodeMirror').CodeMirror;
-    instance.setValue(JSON.stringify([{ foo: 'bar' }]));
+    codemirror().setValue(JSON.stringify([{ foo: 'bar' }]));
     await settled();
     assert
       .dom('[data-test-message-error]')
