@@ -10,12 +10,11 @@ export default {
   ...fields,
   header: text('[data-test-mount-form-header]'),
   submit: clickable('[data-test-mount-submit]'),
-  next: clickable('[data-test-mount-next]'),
   back: clickable('[data-test-mount-back]'),
   path: fillable('[data-test-input="path"]'),
   toggleOptions: clickable('[data-test-toggle-group="Method Options"]'),
   pathValue: value('[data-test-input="path"]'),
-  types: collection('[data-test-mount-type-radio] input', {
+  types: collection('[data-test-mount-type]', {
     select: clickable(),
     id: attribute('id'),
   }),
@@ -26,9 +25,9 @@ export default {
   async mount(type, path) {
     await this.selectType(type);
     if (path) {
-      await this.next().path(path).submit();
+      await this.path(path).submit();
     } else {
-      await this.next().submit();
+      await this.submit();
     }
   },
 };
