@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import { create, clickable, fillable, visitable } from 'ember-cli-page-object';
 import ListView from 'vault/tests/pages/components/list-view';
 
@@ -5,9 +10,9 @@ export default create({
   ...ListView,
   visit: visitable('/vault/secrets/:backend/list?tab=roles'),
   visitCreate: visitable('/vault/secrets/:backend/create?itemType=role'),
-  createLink: clickable('[data-test-secret-create="true"]'),
+  createLink: clickable('[data-test-secret-create]'),
   name: fillable('[data-test-input="name"]'),
   transformations: fillable('[data-test-input="transformations"'),
-  submit: clickable('[data-test-role-transform-create="true"]'),
+  submit: clickable('[data-test-role-transform-create]'),
   modalConfirm: clickable('[data-test-edit-confirm-button]'),
 });
