@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import { create, clickable, fillable, visitable } from 'ember-cli-page-object';
 import ListView from 'vault/tests/pages/components/list-view';
 
@@ -6,7 +11,7 @@ export default create({
   visit: visitable('/vault/secrets/:backend/list'),
   visitShow: visitable('/vault/secrets/:backend/show/:id'),
   visitCreate: visitable('/vault/secrets/:backend/create'),
-  createLink: clickable('[data-test-secret-create="true"]'),
+  createLink: clickable('[data-test-secret-create]'),
   name: fillable('[data-test-input="name"]'),
   submit: clickable('[data-test-transform-create]'),
   type: fillable('[data-test-input="type"'),
