@@ -12,10 +12,8 @@ import { module, test } from 'qunit';
 */
 module('Unit | Utility | timestamp', function () {
   test('it can be overridden', function (assert) {
-    const stub = sinon.stub(timestamp, 'now').callsFake(() => new Date('2030-03-03T03:30:03'));
+    sinon.replace(timestamp, 'now', sinon.fake.returns(new Date('2030-03-03T03:30:03')));
     const result = timestamp.now();
     assert.strictEqual(result.toISOString(), new Date('2030-03-03T03:30:03').toISOString());
-    // Always make sure to restore the stub
-    stub.restore(); // timestamp.now.restore(); also works
   });
 });
