@@ -1,18 +1,21 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package vault
 
 import (
 	"context"
 	"fmt"
 	"reflect"
+	"slices"
 	"sort"
 	"strings"
 
 	"github.com/armon/go-radix"
-	"github.com/hashicorp/errwrap"
 	"github.com/hashicorp/go-multierror"
+	"github.com/hashicorp/go-secure-stdlib/strutil"
 	"github.com/hashicorp/vault/helper/identity"
 	"github.com/hashicorp/vault/helper/namespace"
-	"github.com/hashicorp/vault/sdk/helper/strutil"
 	"github.com/hashicorp/vault/sdk/logical"
 	"github.com/mitchellh/copystructure"
 )
@@ -41,20 +44,27 @@ type PolicyCheckOpts struct {
 }
 
 type AuthResults struct {
-	ACLResults  *ACLResults
-	Allowed     bool
-	RootPrivs   bool
-	DeniedError bool
-	Error       *multierror.Error
+	ACLResults      *ACLResults
+	SentinelResults *SentinelResults
+	Allowed         bool
+	RootPrivs       bool
+	DeniedError     bool
+	Error           *multierror.Error
 }
 
 type ACLResults struct {
-	Allowed            bool
-	RootPrivs          bool
-	IsRoot             bool
-	MFAMethods         []string
-	ControlGroup       *ControlGroup
-	CapabilitiesBitmap uint32
+	Allowed             bool
+	RootPrivs           bool
+	IsRoot              bool
+	MFAMethods          []string
+	ControlGroup        *ControlGroup
+	CapabilitiesBitmap  uint32
+	GrantingPolicies    []logical.PolicyInfo
+	SubscribeEventTypes []string
+}
+
+type SentinelResults struct {
+	GrantingPolicies []logical.PolicyInfo
 }
 
 // NewACL is used to construct a policy based ACL from a set of policies.
@@ -125,8 +135,12 @@ func NewACL(ctx context.Context, policies []*Policy) (*ACL, error) {
 			if !ok {
 				clonedPerms, err := pc.Permissions.Clone()
 				if err != nil {
-					return nil, errwrap.Wrapf("error cloning ACL permissions: {{err}}", err)
+					return nil, fmt.Errorf("error cloning ACL permissions: %w", err)
 				}
+
+				// Store this policy name as the policy that permits these
+				// capabilities
+				clonedPerms.GrantingPoliciesMap = addGrantingPoliciesToMap(nil, policy, clonedPerms.CapabilitiesBitmap)
 				switch {
 				case pc.HasSegmentWildcards:
 					a.segmentWildcardPaths[pc.Path] = clonedPerms
@@ -156,6 +170,7 @@ func NewACL(ctx context.Context, policies []*Policy) (*ACL, error) {
 				// Insert the capabilities in this new policy into the existing
 				// value
 				existingPerms.CapabilitiesBitmap = existingPerms.CapabilitiesBitmap | pc.Permissions.CapabilitiesBitmap
+				existingPerms.GrantingPoliciesMap = addGrantingPoliciesToMap(existingPerms.GrantingPoliciesMap, policy, pc.Permissions.CapabilitiesBitmap)
 			}
 
 			// Note: In these stanzas, we're preferring minimum lifetimes. So
@@ -240,26 +255,33 @@ func NewACL(ctx context.Context, policies []*Policy) (*ACL, error) {
 				if existingPerms.MFAMethods == nil {
 					existingPerms.MFAMethods = pc.Permissions.MFAMethods
 				} else {
-					for _, method := range pc.Permissions.MFAMethods {
-						existingPerms.MFAMethods = append(existingPerms.MFAMethods, method)
-					}
+					existingPerms.MFAMethods = append(existingPerms.MFAMethods, pc.Permissions.MFAMethods...)
 				}
 				existingPerms.MFAMethods = strutil.RemoveDuplicates(existingPerms.MFAMethods, false)
 			}
 
-			// No need to dedupe this list since any authorization can satisfy any factor
+			// No need to dedupe this list since any authorization can satisfy any factor, so long as
+			// the factor matches the specified permission requested.
 			if pc.Permissions.ControlGroup != nil {
 				if len(pc.Permissions.ControlGroup.Factors) > 0 {
 					if existingPerms.ControlGroup == nil {
-						existingPerms.ControlGroup = pc.Permissions.ControlGroup
-					} else {
-						for _, authz := range pc.Permissions.ControlGroup.Factors {
-							existingPerms.ControlGroup.Factors = append(existingPerms.ControlGroup.Factors, authz)
+						cg, err := pc.Permissions.ControlGroup.Clone()
+						if err != nil {
+							return nil, err
 						}
+						existingPerms.ControlGroup = cg
+					} else {
+						existingPerms.ControlGroup.Factors = append(existingPerms.ControlGroup.Factors, pc.Permissions.ControlGroup.Factors...)
 					}
 				}
 			}
 
+			if len(pc.Permissions.SubscribeEventTypes) > 0 {
+				if len(existingPerms.SubscribeEventTypes) > 0 {
+					existingPerms.SubscribeEventTypes = strutil.RemoveDuplicates(append(existingPerms.SubscribeEventTypes, pc.Permissions.SubscribeEventTypes...), false)
+				}
+			}
+
 		INSERT:
 			switch {
 			case pc.HasSegmentWildcards:
@@ -272,7 +294,7 @@ func NewACL(ctx context.Context, policies []*Policy) (*ACL, error) {
 	return a, nil
 }
 
-func (a *ACL) Capabilities(ctx context.Context, path string) (pathCapabilities []string) {
+func (a *ACL) CapabilitiesAndSubscribeEventTypes(ctx context.Context, path string) (pathCapabilities []string, subscribeEventTypes []string) {
 	req := &logical.Request{
 		Path: path,
 		// doesn't matter, but use List to trigger fallback behavior so we can
@@ -282,9 +304,9 @@ func (a *ACL) Capabilities(ctx context.Context, path string) (pathCapabilities [
 
 	res := a.AllowOperation(ctx, req, true)
 	if res.IsRoot {
-		return []string{RootCapability}
+		return []string{RootCapability}, []string{"*"}
 	}
-
+	subscribeEventTypes = res.SubscribeEventTypes
 	capabilities := res.CapabilitiesBitmap
 
 	if capabilities&SudoCapabilityInt > 0 {
@@ -305,15 +327,27 @@ func (a *ACL) Capabilities(ctx context.Context, path string) (pathCapabilities [
 	if capabilities&CreateCapabilityInt > 0 {
 		pathCapabilities = append(pathCapabilities, CreateCapability)
 	}
+	if capabilities&PatchCapabilityInt > 0 {
+		pathCapabilities = append(pathCapabilities, PatchCapability)
+	}
+	if capabilities&SubscribeCapabilityInt > 0 {
+		pathCapabilities = append(pathCapabilities, SubscribeCapability)
+	}
 
 	// If "deny" is explicitly set or if the path has no capabilities at all,
 	// set the path capabilities to "deny"
 	if capabilities&DenyCapabilityInt > 0 || len(pathCapabilities) == 0 {
 		pathCapabilities = []string{DenyCapability}
 	}
+
 	return
 }
 
+func (a *ACL) Capabilities(ctx context.Context, path string) []string {
+	pathCapabilities, _ := a.CapabilitiesAndSubscribeEventTypes(ctx, path)
+	return pathCapabilities
+}
+
 // AllowOperation is used to check if the given operation is permitted.
 func (a *ACL) AllowOperation(ctx context.Context, req *logical.Request, capCheckOnly bool) (ret *ACLResults) {
 	ret = new(ACLResults)
@@ -323,6 +357,13 @@ func (a *ACL) AllowOperation(ctx context.Context, req *logical.Request, capCheck
 		ret.Allowed = true
 		ret.RootPrivs = true
 		ret.IsRoot = true
+		ret.GrantingPolicies = []logical.PolicyInfo{{
+			Name:          "root",
+			NamespaceId:   "root",
+			NamespacePath: "",
+			Type:          "acl",
+		}}
+		ret.SubscribeEventTypes = []string{"*"}
 		return
 	}
 	op := req.Operation
@@ -368,6 +409,16 @@ func (a *ACL) AllowOperation(ctx context.Context, req *logical.Request, capCheck
 		}
 	}
 
+	// List operations need to check without the trailing slash first, because
+	// there could be other rules with trailing wildcards that will match the
+	// path
+	if op == logical.ListOperation && strings.HasSuffix(path, "/") {
+		permissions = a.CheckAllowedFromNonExactPaths(strings.TrimSuffix(path, "/"), false)
+		if permissions != nil {
+			capabilities = permissions.CapabilitiesBitmap
+			goto CHECK
+		}
+	}
 	permissions = a.CheckAllowedFromNonExactPaths(path, false)
 	if permissions != nil {
 		capabilities = permissions.CapabilitiesBitmap
@@ -388,29 +439,40 @@ CHECK:
 	// rather than policy root
 	if capCheckOnly {
 		ret.CapabilitiesBitmap = capabilities
+		ret.SubscribeEventTypes = slices.Clone(permissions.SubscribeEventTypes)
 		return ret
 	}
 
 	ret.MFAMethods = permissions.MFAMethods
 	ret.ControlGroup = permissions.ControlGroup
 
+	var grantingPolicies []logical.PolicyInfo
 	operationAllowed := false
 	switch op {
 	case logical.ReadOperation:
 		operationAllowed = capabilities&ReadCapabilityInt > 0
+		grantingPolicies = permissions.GrantingPoliciesMap[ReadCapabilityInt]
 	case logical.ListOperation:
 		operationAllowed = capabilities&ListCapabilityInt > 0
+		grantingPolicies = permissions.GrantingPoliciesMap[ListCapabilityInt]
 	case logical.UpdateOperation:
 		operationAllowed = capabilities&UpdateCapabilityInt > 0
+		grantingPolicies = permissions.GrantingPoliciesMap[UpdateCapabilityInt]
 	case logical.DeleteOperation:
 		operationAllowed = capabilities&DeleteCapabilityInt > 0
+		grantingPolicies = permissions.GrantingPoliciesMap[DeleteCapabilityInt]
 	case logical.CreateOperation:
 		operationAllowed = capabilities&CreateCapabilityInt > 0
+		grantingPolicies = permissions.GrantingPoliciesMap[CreateCapabilityInt]
+	case logical.PatchOperation:
+		operationAllowed = capabilities&PatchCapabilityInt > 0
+		grantingPolicies = permissions.GrantingPoliciesMap[PatchCapabilityInt]
 
 	// These three re-use UpdateCapabilityInt since that's the most appropriate
 	// capability/operation mapping
 	case logical.RevokeOperation, logical.RenewOperation, logical.RollbackOperation:
 		operationAllowed = capabilities&UpdateCapabilityInt > 0
+		grantingPolicies = permissions.GrantingPoliciesMap[UpdateCapabilityInt]
 
 	default:
 		return
@@ -420,6 +482,8 @@ CHECK:
 		return
 	}
 
+	ret.GrantingPolicies = grantingPolicies
+
 	if permissions.MaxWrappingTTL > 0 {
 		if req.WrapInfo == nil || req.WrapInfo.TTL > permissions.MaxWrappingTTL {
 			return
@@ -440,7 +504,7 @@ CHECK:
 
 	// Only check parameter permissions for operations that can modify
 	// parameters.
-	if op == logical.ReadOperation || op == logical.UpdateOperation || op == logical.CreateOperation {
+	if op == logical.ReadOperation || op == logical.UpdateOperation || op == logical.CreateOperation || op == logical.PatchOperation {
 		for _, parameter := range permissions.RequiredParameters {
 			if _, ok := req.Data[strings.ToLower(parameter)]; !ok {
 				return
@@ -681,14 +745,16 @@ func (c *Core) performPolicyChecks(ctx context.Context, acl *ACL, te *logical.To
 		ret.RootPrivs = ret.ACLResults.RootPrivs
 		// Root is always allowed; skip Sentinel/MFA checks
 		if ret.ACLResults.IsRoot {
-			//logger.Warn("token is root, skipping checks")
+			// logger.Warn("token is root, skipping checks")
 			ret.Allowed = true
 			return ret
 		}
 		if !ret.ACLResults.Allowed {
 			return ret
 		}
-		if !ret.RootPrivs && opts.RootPrivsRequired {
+		// Since HelpOperation was fast-pathed inside AllowOperation, RootPrivs will not have been populated in this
+		// case, so we need to special-case that here as well, or we'll block HelpOperation on all sudo-protected paths.
+		if !ret.RootPrivs && opts.RootPrivsRequired && req.Operation != logical.HelpOperation {
 			return ret
 		}
 	}
