@@ -8,11 +8,10 @@ Example of how to use docker.NewReplicationSetDocker(t), assuming
 you point VAULT_BINARY to an Enterprise Vault binary:
 
 import (
-	"context"
-	"testing"
-	"time"
 
-	"github.com/hashicorp/vault/sdk/helper/testcluster/docker"
+
+
+
 )
 
 // TestStandardPerfReplication_Docker tests that we can create two 3-node
