@@ -801,7 +801,7 @@ func runAutoseal(t *testing.T, logger hclog.Logger, storage teststorage.Reusable
 			t.Fatal(err)
 		}
 	} else {
-		if err := cluster.UnsealCoresWithError(true); err != nil {
+		if err := cluster.UnsealCoresWithError(t, true); err != nil {
 			t.Fatal(err)
 		}
 	}
