@@ -110,6 +110,7 @@ type client struct {
 	ClientSecret string `json:"client_secret"`
 }
 
+//go:generate enumer -type=clientType -trimprefix=clientType -transform=snake
 type clientType int
 
 const (
@@ -117,17 +118,6 @@ const (
 	public
 )
 
-func (k clientType) String() string {
-	switch k {
-	case confidential:
-		return "confidential"
-	case public:
-		return "public"
-	default:
-		return "unknown"
-	}
-}
-
 type provider struct {
 	Issuer           string   `json:"issuer"`
 	AllowedClientIDs []string `json:"allowed_client_ids"`
@@ -2629,10 +2619,6 @@ func (i *IdentityStore) lazyGenerateDefaultKey(ctx context.Context, storage logi
 			return err
 		}
 
-		if err := i.oidcCache.Delete(ns, namedKeyCachePrefix+defaultKeyName); err != nil {
-			return err
-		}
-
 		entry, err := logical.StorageEntryJSON(namedKeyConfigPath+defaultKeyName, defaultKey)
 		if err != nil {
 			return err
@@ -2640,6 +2626,10 @@ func (i *IdentityStore) lazyGenerateDefaultKey(ctx context.Context, storage logi
 		if err := storage.Put(ctx, entry); err != nil {
 			return err
 		}
+
+		if err := i.oidcCache.Flush(ns); err != nil {
+			return err
+		}
 	}
 
 	return nil
