@@ -8,29 +8,27 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/hashicorp/vault/helper/namespace"
-	"github.com/hashicorp/vault/helper/versions"
-
 	"github.com/hashicorp/go-multierror"
 	"github.com/hashicorp/go-secure-stdlib/strutil"
+	"github.com/hashicorp/vault/helper/namespace"
 	"github.com/hashicorp/vault/sdk/helper/consts"
 	"github.com/hashicorp/vault/sdk/logical"
 	"github.com/hashicorp/vault/sdk/plugin"
 )
 
+const (
+	pluginReloadPluginsType = "plugins"
+	pluginReloadMountsType  = "mounts"
+)
+
 // reloadMatchingPluginMounts reloads provided mounts, regardless of
 // plugin name, as long as the backend type is plugin.
-func (c *Core) reloadMatchingPluginMounts(ctx context.Context, mounts []string) error {
+func (c *Core) reloadMatchingPluginMounts(ctx context.Context, ns *namespace.Namespace, mounts []string) error {
 	c.mountsLock.RLock()
 	defer c.mountsLock.RUnlock()
 	c.authLock.RLock()
 	defer c.authLock.RUnlock()
 
-	ns, err := namespace.FromContext(ctx)
-	if err != nil {
-		return err
-	}
-
 	var errors error
 	for _, mount := range mounts {
 		var isAuth bool
@@ -65,57 +63,99 @@ func (c *Core) reloadMatchingPluginMounts(ctx context.Context, mounts []string)
 			errors = multierror.Append(errors, fmt.Errorf("cannot reload plugin on %q: %w", mount, err))
 			continue
 		}
-		c.logger.Info("successfully reloaded plugin", "plugin", entry.Accessor, "path", entry.Path, "version", entry.Version)
+		c.logger.Info("successfully reloaded plugin", "plugin", entry.Accessor, "path", entry.Path, "version", entry.RunningVersion)
 	}
 	return errors
 }
 
-// reloadPlugin reloads all mounted backends that are of
-// plugin pluginName (name of the plugin as registered in
-// the plugin catalog).
-func (c *Core) reloadMatchingPlugin(ctx context.Context, pluginName string) error {
-	c.mountsLock.RLock()
-	defer c.mountsLock.RUnlock()
-	c.authLock.RLock()
-	defer c.authLock.RUnlock()
-
-	ns, err := namespace.FromContext(ctx)
-	if err != nil {
-		return err
+// reloadMatchingPlugin reloads all mounted backends that are named pluginName
+// (name of the plugin as registered in the plugin catalog). It returns the
+// number of plugins that were reloaded and an error if any.
+func (c *Core) reloadMatchingPlugin(ctx context.Context, ns *namespace.Namespace, pluginType consts.PluginType, pluginName string) (reloaded int, err error) {
+	var secrets, auth, database bool
+	switch pluginType {
+	case consts.PluginTypeSecrets:
+		secrets = true
+	case consts.PluginTypeCredential:
+		auth = true
+	case consts.PluginTypeDatabase:
+		database = true
+	case consts.PluginTypeUnknown:
+		secrets = true
+		auth = true
+		database = true
+	default:
+		return reloaded, fmt.Errorf("unsupported plugin type %q", pluginType.String())
 	}
 
-	// Filter mount entries that only matches the plugin name
-	for _, entry := range c.mounts.Entries {
-		// We dont reload mounts that are not in the same namespace
-		if ns.ID != entry.Namespace().ID {
-			continue
-		}
-		if entry.Type == pluginName || (entry.Type == "plugin" && entry.Config.PluginName == pluginName) {
-			err := c.reloadBackendCommon(ctx, entry, false)
-			if err != nil {
-				return err
+	if secrets || database {
+		c.mountsLock.RLock()
+		defer c.mountsLock.RUnlock()
+
+		for _, entry := range c.mounts.Entries {
+			// We don't reload mounts that are not in the same namespace
+			if ns != nil && ns.ID != entry.Namespace().ID {
+				continue
 			}
-			c.logger.Info("successfully reloaded plugin", "plugin", pluginName, "path", entry.Path, "version", entry.Version)
-		}
-	}
 
-	// Filter auth mount entries that ony matches the plugin name
-	for _, entry := range c.auth.Entries {
-		// We dont reload mounts that are not in the same namespace
-		if ns.ID != entry.Namespace().ID {
-			continue
-		}
+			if secrets && (entry.Type == pluginName || (entry.Type == "plugin" && entry.Config.PluginName == pluginName)) {
+				err := c.reloadBackendCommon(ctx, entry, false)
+				if err != nil {
+					return reloaded, err
+				}
+				reloaded++
+				c.logger.Info("successfully reloaded plugin", "plugin", pluginName, "namespace", entry.Namespace(), "path", entry.Path, "version", entry.RunningVersion)
+			} else if database && entry.Type == "database" {
+				// The combined database plugin is itself a secrets engine, but
+				// knowledge of whether a database plugin is in use within a particular
+				// mount is internal to the combined database plugin's storage, so
+				// we delegate the reload request with an internally routed request.
+				reqCtx := namespace.ContextWithNamespace(ctx, entry.namespace)
+				req := &logical.Request{
+					Operation: logical.UpdateOperation,
+					Path:      entry.Path + "reload/" + pluginName,
+				}
+				resp, err := c.router.Route(reqCtx, req)
+				if err != nil {
+					return reloaded, err
+				}
+				if resp == nil {
+					return reloaded, fmt.Errorf("failed to reload %q database plugin(s) mounted under %s", pluginName, entry.Path)
+				}
+				if resp.IsError() {
+					return reloaded, fmt.Errorf("failed to reload %q database plugin(s) mounted under %s: %s", pluginName, entry.Path, resp.Error())
+				}
 
-		if entry.Type == pluginName || (entry.Type == "plugin" && entry.Config.PluginName == pluginName) {
-			err := c.reloadBackendCommon(ctx, entry, true)
-			if err != nil {
-				return err
+				if count, ok := resp.Data["count"].(int); ok && count > 0 {
+					c.logger.Info("successfully reloaded database plugin(s)", "plugin", pluginName, "namespace", entry.Namespace(), "path", entry.Path, "connections", resp.Data["connections"])
+					reloaded += count
+				}
 			}
-			c.logger.Info("successfully reloaded plugin", "plugin", entry.Accessor, "path", entry.Path, "version", entry.Version)
 		}
 	}
 
-	return nil
+	if auth {
+		c.authLock.RLock()
+		defer c.authLock.RUnlock()
+
+		for _, entry := range c.auth.Entries {
+			// We don't reload mounts that are not in the same namespace
+			if ns != nil && ns.ID != entry.Namespace().ID {
+				continue
+			}
+
+			if entry.Type == pluginName || (entry.Type == "plugin" && entry.Config.PluginName == pluginName) {
+				err := c.reloadBackendCommon(ctx, entry, true)
+				if err != nil {
+					return reloaded, err
+				}
+				reloaded++
+				c.logger.Info("successfully reloaded plugin", "plugin", entry.Accessor, "path", entry.Path, "version", entry.RunningVersion)
+			}
+		}
+	}
+
+	return reloaded, nil
 }
 
 // reloadBackendCommon is a generic method to reload a backend provided a
@@ -182,9 +222,9 @@ func (c *Core) reloadBackendCommon(ctx context.Context, entry *MountEntry, isAut
 	oldSha := entry.RunningSha256
 	if !isAuth {
 		// Dispense a new backend
-		backend, entry.RunningSha256, err = c.newLogicalBackend(ctx, entry, sysView, view)
+		backend, err = c.newLogicalBackend(ctx, entry, sysView, view)
 	} else {
-		backend, entry.RunningSha256, err = c.newCredentialBackend(ctx, entry, sysView, view)
+		backend, err = c.newCredentialBackend(ctx, entry, sysView, view)
 	}
 	if err != nil {
 		return err
@@ -193,19 +233,6 @@ func (c *Core) reloadBackendCommon(ctx context.Context, entry *MountEntry, isAut
 		return fmt.Errorf("nil backend of type %q returned from creation function", entry.Type)
 	}
 
-	// update the entry running version with the configured version, which was verified during registration.
-	entry.RunningVersion = entry.Version
-	if entry.RunningVersion == "" {
-		// don't set the running version to a builtin if it is running as an external plugin
-		if entry.RunningSha256 == "" {
-			if isAuth {
-				entry.RunningVersion = versions.GetBuiltinVersion(consts.PluginTypeCredential, entry.Type)
-			} else {
-				entry.RunningVersion = versions.GetBuiltinVersion(consts.PluginTypeSecrets, entry.Type)
-			}
-		}
-	}
-
 	// update the mount table since we changed the runningSha
 	if oldSha != entry.RunningSha256 && MountTableUpdateStorage {
 		if isAuth {
@@ -249,6 +276,11 @@ func (c *Core) reloadBackendCommon(ctx context.Context, entry *MountEntry, isAut
 				return err
 			}
 			re.loginPaths.Store(loginPathsEntry)
+			binaryPathsEntry, err := parseUnauthenticatedPaths(paths.Binary)
+			if err != nil {
+				return err
+			}
+			re.binaryPaths.Store(binaryPathsEntry)
 		}
 	}
 
