@@ -9,9 +9,8 @@ import (
 	"context"
 
 	log "github.com/hashicorp/go-hclog"
-	"github.com/hashicorp/vault/helper/metricsutil"
-
 	"github.com/hashicorp/go-memdb"
+	"github.com/hashicorp/vault/helper/metricsutil"
 )
 
 func quotaTypes() []string {
@@ -32,9 +31,14 @@ func (m *Manager) inLeasePathCache(path string) bool {
 	return false
 }
 
+func (m *Manager) setupDefaultLeaseCountQuotaInStorage(_ctx context.Context) error {
+	return nil
+}
+
 type entManager struct {
 	isPerfStandby bool
 	isDRSecondary bool
+	isNewInstall  bool
 }
 
 func (*entManager) Reset() error {
@@ -43,6 +47,10 @@ func (*entManager) Reset() error {
 
 type LeaseCountQuota struct{}
 
+func (l LeaseCountQuota) GetNamespacePath() string {
+	panic("implement me")
+}
+
 func (l LeaseCountQuota) IsInheritable() bool {
 	panic("implement me")
 }
