@@ -3,7 +3,9 @@
 
 package vault
 
-import "context"
+import (
+	"context"
+)
 
 // RouterAccess provides access into some things necessary for testing
 type RouterAccess struct {
@@ -17,3 +19,11 @@ func NewRouterAccess(c *Core) *RouterAccess {
 func (r *RouterAccess) StoragePrefixByAPIPath(ctx context.Context, path string) (string, bool) {
 	return r.c.router.MatchingStoragePrefixByAPIPath(ctx, path)
 }
+
+func (r *RouterAccess) IsBinaryPath(ctx context.Context, path string) bool {
+	return r.c.router.BinaryPath(ctx, path)
+}
+
+func (r *RouterAccess) IsLimitedPath(ctx context.Context, path string) bool {
+	return r.c.router.LimitedPath(ctx, path)
+}
