@@ -8,6 +8,7 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"os"
 	"reflect"
 	"sort"
 	"strings"
@@ -15,15 +16,12 @@ import (
 	"sync/atomic"
 	"time"
 
-	"github.com/google/go-cmp/cmp"
-
-	"github.com/hashicorp/go-hclog"
-
-	"github.com/hashicorp/vault/internalshared/configutil"
-
 	metrics "github.com/armon/go-metrics"
+	"github.com/google/go-cmp/cmp"
+	"github.com/hashicorp/go-hclog"
 	wrapping "github.com/hashicorp/go-kms-wrapping/v2"
 	"github.com/hashicorp/go-kms-wrapping/v2/aead"
+	"github.com/hashicorp/vault/internalshared/configutil"
 )
 
 type StoredKeysSupport int
@@ -37,8 +35,11 @@ const (
 )
 
 var (
-	ErrUnconfiguredWrapper = errors.New("unconfigured wrapper")
-	ErrNoHealthySeals      = errors.New("no healthy seals!")
+	ErrUnconfiguredWrapper  = errors.New("unconfigured wrapper")
+	ErrNoHealthySeals       = errors.New("no healthy seals!")
+	ErrNoConfiguredSeals    = errors.New("no configured seals")
+	ErrNoSealGenerationInfo = errors.New("no seal generation info")
+	ErrNoSeals              = errors.New("no seals provided in the configuration")
 )
 
 func (s StoredKeysSupport) String() string {
@@ -58,6 +59,7 @@ type SealGenerationInfo struct {
 	Generation uint64
 	Seals      []*configutil.KMS
 	rewrapped  atomic.Bool
+	Enabled    bool
 }
 
 // Validate is used to sanity check the seal generation info being created
@@ -80,6 +82,15 @@ func (sgi *SealGenerationInfo) Validate(existingSgi *SealGenerationInfo, hasPart
 		return nil
 	}
 
+	// Validate that we're in a safe spot with respect to disabling multiseal
+	if existingSgi.Enabled && !sgi.Enabled {
+		if len(existingSgi.Seals) > 1 {
+			return fmt.Errorf("multi-seal is disabled but previous configuration had multiple seals.  re-enable and migrate to a single seal before disabling multi-seal")
+		} else if !existingSgi.IsRewrapped() {
+			return fmt.Errorf("multi-seal is disabled but previous storage was not fully re-wrapped, re-enable multi-seal and allow rewrapping to complete before disabling multi-seal")
+		}
+	}
+
 	existingSealNameAndType := sealNameAndTypeAsStr(existingSgi.Seals)
 	previousShamirConfigured := false
 
@@ -100,7 +111,7 @@ func (sgi *SealGenerationInfo) Validate(existingSgi *SealGenerationInfo, hasPart
 		}
 	}
 
-	if !previousShamirConfigured && (!existingSgi.IsRewrapped() || hasPartiallyWrappedPaths) {
+	if !previousShamirConfigured && (!existingSgi.IsRewrapped() || hasPartiallyWrappedPaths) && os.Getenv("VAULT_SEAL_REWRAP_SAFETY") != "disable" {
 		return errors.New("cannot make seal config changes while seal re-wrap is in progress, please revert any seal configuration changes")
 	}
 
@@ -233,6 +244,7 @@ type sealGenerationInfoJson struct {
 	Generation uint64
 	Seals      []*configutil.KMS
 	Rewrapped  bool
+	Enabled    bool
 }
 
 func (sgi *SealGenerationInfo) MarshalJSON() ([]byte, error) {
@@ -240,6 +252,7 @@ func (sgi *SealGenerationInfo) MarshalJSON() ([]byte, error) {
 		Generation: sgi.Generation,
 		Seals:      sgi.Seals,
 		Rewrapped:  sgi.IsRewrapped(),
+		Enabled:    sgi.Enabled,
 	})
 }
 
@@ -252,6 +265,7 @@ func (sgi *SealGenerationInfo) UnmarshalJSON(b []byte) error {
 	sgi.Generation = value.Generation
 	sgi.Seals = value.Seals
 	sgi.SetRewrapped(value.Rewrapped)
+	sgi.Enabled = value.Enabled
 
 	return nil
 }
@@ -323,15 +337,17 @@ type access struct {
 
 var _ Access = (*access)(nil)
 
-func NewAccess(logger hclog.Logger, sealGenerationInfo *SealGenerationInfo, sealWrappers []*SealWrapper) Access {
+func NewAccess(logger hclog.Logger, sealGenerationInfo *SealGenerationInfo, sealWrappers []*SealWrapper) (Access, error) {
 	if logger == nil {
 		logger = hclog.NewNullLogger()
 	}
 	if sealGenerationInfo == nil {
-		panic("cannot create a seal.Access without a SealGenerationInfo")
+		logger.Error("cannot create a seal.Access without a SealGenerationInfo")
+		return nil, ErrNoSealGenerationInfo
 	}
 	if len(sealWrappers) == 0 {
-		panic("cannot create a seal.Access without any seal wrappers")
+		logger.Error("cannot create a seal.Access without any seal wrappers")
+		return nil, ErrNoSeals
 	}
 	a := &access{
 		sealGenerationInfo: sealGenerationInfo,
@@ -342,9 +358,15 @@ func NewAccess(logger hclog.Logger, sealGenerationInfo *SealGenerationInfo, seal
 		a.wrappersByPriority[i] = sw
 	}
 
+	configuredSealWrappers := a.GetConfiguredSealWrappersByPriority()
+	if len(configuredSealWrappers) == 0 {
+		a.logger.Error("cannot create a seal.Access without any configured seal wrappers")
+		return nil, ErrNoConfiguredSeals
+	}
+
 	sort.Slice(a.wrappersByPriority, func(i int, j int) bool { return a.wrappersByPriority[i].Priority < a.wrappersByPriority[j].Priority })
 
-	return a
+	return a, nil
 }
 
 func NewAccessFromSealWrappers(logger hclog.Logger, generation uint64, rewrapped bool, sealWrappers []*SealWrapper) (Access, error) {
@@ -364,7 +386,7 @@ func NewAccessFromSealWrappers(logger hclog.Logger, generation uint64, rewrapped
 			Name:     sw.Name,
 		})
 	}
-	return NewAccess(logger, sealGenerationInfo, sealWrappers), nil
+	return NewAccess(logger, sealGenerationInfo, sealWrappers)
 }
 
 // NewAccessFromWrapper creates an enabled Access for a single wrapping.Wrapper.
@@ -403,6 +425,10 @@ func healthyWrappers(wrapper *SealWrapper) bool {
 	return wrapper.IsHealthy()
 }
 
+func unhealthyWrappers(wrapper *SealWrapper) bool {
+	return !wrapper.IsHealthy()
+}
+
 func enabledWrappers(wrapper *SealWrapper) bool {
 	return !wrapper.Disabled
 }
@@ -481,11 +507,17 @@ func (a *access) IsUpToDate(ctx context.Context, value *MultiWrapValue, forceKey
 			return false, JoinSealWrapErrors("cannot determine key IDs of Access wrappers", errs)
 		}
 		if len(errs) > 0 {
-			msg := "could not determine key IDs of some Access wrappers"
-			a.logger.Error("partial failure refreshing seal key IDs", "err", JoinSealWrapErrors(msg, errs))
-			return false, JoinSealWrapErrors(msg, errs)
+			a.logger.Warn("cannot determine if seal wrapped entry needs update: there were errors determining the key IDs for one or more seals")
+			a.logger.Debug("cannot determine if seal wrapped entry needs update", "err", JoinSealWrapErrors("error refreshing key IDs of Access wrappers", errs))
+
+			// Return true, since the encrypted values cannot be re-encrypted without
+			// losing the ciphertext of unhealthy wrappers.
+			return true, nil
 		}
-		a.keyIdSet.set(test)
+	} else if !a.keyIdSet.initialized() {
+		// Since the key ID set is not initialized, we cannot determine if the value is up-to-date, so assume it is.
+		// Note that we cannot just force an update, since that breaks migrations to a Shamir defaultSeal.
+		return true, nil
 	}
 
 	return a.keyIdSet.equal(value), nil
@@ -504,10 +536,30 @@ const (
 
 // Encrypt uses the underlying seal to encrypt the plaintext and returns it.
 func (a *access) Encrypt(ctx context.Context, plaintext []byte, options ...wrapping.Option) (*MultiWrapValue, map[string]error) {
+	errs := make(map[string]error)
+
 	// Note that we do not encrypt with disabled wrappers. Disabled wrappers are only used to decrypt.
 	candidateWrappers := a.filterSealWrappers(enabledWrappers, healthyWrappers)
-	if len(candidateWrappers) == 0 {
-		// If all seals are unhealthy, try any way since a seal may have recovered
+	if len(candidateWrappers) > 0 {
+		// As there are healthy wrappers, add errors for any unhealthy ones, so that it
+		// it is clear that the resulting MultiWrapValue is missing ciphertext for some seals.
+		for i, unhealthyWrapper := range a.filterSealWrappers(enabledWrappers, unhealthyWrappers) {
+			var keyId string
+			if unhealthyWrapper.Wrapper != nil {
+				// Annoying, apparently Wrapper may be null, see setSeal() in server.go,
+				// in the config seal loop.
+				keyId, _ = unhealthyWrapper.Wrapper.KeyId(ctx)
+			}
+			if keyId == "" {
+				keyId = unhealthyWrapper.Name
+				if _, duplicated := errs[keyId]; duplicated {
+					keyId = fmt.Sprintf("%s-%d", keyId, i)
+				}
+			}
+			errs[keyId] = errors.New("seal is unhealthy")
+		}
+	} else {
+		// If all seals are unhealthy, try with all of them since a seal may have recovered.
 		candidateWrappers = a.filterSealWrappers(enabledWrappers)
 	}
 	enabledWrappersByPriority := filterSealWrappers(candidateWrappers, configuredWrappers)
@@ -581,7 +633,6 @@ GATHER_RESULTS:
 
 	// Sort out the successful results from the errors
 	var slots []*wrapping.BlobInfo
-	errs := make(map[string]error)
 	for _, sealWrapper := range enabledWrappersByPriority {
 		if result, ok := results[sealWrapper.Name]; ok {
 			if result.err != nil {
@@ -632,18 +683,17 @@ GATHER_RESULTS:
 func (a *access) tryEncrypt(ctx context.Context, sealWrapper *SealWrapper, plaintext []byte, options ...wrapping.Option) (*wrapping.BlobInfo, error) {
 	now := time.Now()
 	var encryptErr error
+	mLabels := []metrics.Label{{Name: "seal_wrapper_name", Value: sealWrapper.Name}}
+
 	defer func(now time.Time) {
-		metrics.MeasureSince([]string{"seal", "encrypt", "time"}, now)
-		metrics.MeasureSince([]string{"seal", sealWrapper.Name, "encrypt", "time"}, now)
+		metrics.MeasureSinceWithLabels([]string{"seal", "encrypt", "time"}, now, mLabels)
 
 		if encryptErr != nil {
-			metrics.IncrCounter([]string{"seal", "encrypt", "error"}, 1)
-			metrics.IncrCounter([]string{"seal", sealWrapper.Name, "encrypt", "error"}, 1)
+			metrics.IncrCounterWithLabels([]string{"seal", "encrypt", "error"}, 1, mLabels)
 		}
 	}(now)
 
-	metrics.IncrCounter([]string{"seal", "encrypt"}, 1)
-	metrics.IncrCounter([]string{"seal", sealWrapper.Name, "encrypt"}, 1)
+	metrics.IncrCounterWithLabels([]string{"seal", "encrypt"}, 1, mLabels)
 
 	ciphertext, encryptErr := sealWrapper.Wrapper.Encrypt(ctx, plaintext, options...)
 	if encryptErr != nil {
@@ -665,8 +715,6 @@ func (a *access) tryEncrypt(ctx context.Context, sealWrapper *SealWrapper, plain
 // Returns the plaintext, a flag indicating whether the ciphertext is up-to-date
 // (according to IsUpToDate), and an error.
 func (a *access) Decrypt(ctx context.Context, ciphertext *MultiWrapValue, options ...wrapping.Option) ([]byte, bool, error) {
-	blobInfoMap := slotsByKeyId(ciphertext)
-
 	isUpToDate, err := a.IsUpToDate(ctx, ciphertext, false)
 	if err != nil {
 		return nil, false, err
@@ -711,30 +759,34 @@ func (a *access) Decrypt(ctx context.Context, ciphertext *MultiWrapValue, option
 	}
 
 	decrypt := func(sealWrapper *SealWrapper) {
-		pt, oldKey, err := a.tryDecrypt(ctx, sealWrapper, blobInfoMap, options)
+		pt, oldKey, err := a.tryDecrypt(ctx, sealWrapper, ciphertext, options)
 		reportResult(sealWrapper.Name, pt, oldKey, err)
 	}
 
 	// Start goroutines to decrypt the value
-
 	first := wrappersByPriority[0]
-	// First, if we only have one slot, try matching by keyId
-	if len(blobInfoMap) == 1 {
-	outer:
-		for k := range blobInfoMap {
-			for _, sealWrapper := range wrappersByPriority {
-				keyId, err := sealWrapper.Wrapper.KeyId(ctx)
-				if err != nil {
-					resultWg.Add(1)
-					go reportResult(sealWrapper.Name, nil, false, err)
-					continue
-				}
-				if keyId == k {
-					first = sealWrapper
-					break outer
-				}
-			}
+	found := false
+outer:
+	// This loop finds the highest priority seal with a keyId in common with the blobInfoMap,
+	// and ensures we'll use it first.  This should equal the highest priority wrapper in the nominal
+	// case, but may not if a seal is unhealthy.  This ensures we try the highest priority healthy
+	// seal first if available, and warn if we don't think we have one in common.
+	for _, sealWrapper := range wrappersByPriority {
+		keyId, err := sealWrapper.Wrapper.KeyId(ctx)
+		if err != nil {
+			resultWg.Add(1)
+			go reportResult(sealWrapper.Name, nil, false, err)
+			continue
 		}
+		if bi := ciphertext.BlobInfoForKeyId(keyId); bi != nil {
+			found = true
+			first = sealWrapper
+			break outer
+		}
+	}
+
+	if !found {
+		a.logger.Warn("while unwrapping, value has no key-id in common with currently healthy seals.  Trying all healthy seals")
 	}
 
 	resultWg.Add(1)
@@ -750,7 +802,7 @@ func (a *access) Decrypt(ctx context.Context, ciphertext *MultiWrapValue, option
 		}
 	}
 
-	// Gathering failures, but return right away if there is a succesful result
+	// Gathering failures, but return right away if there is a successful result
 	errs := make(map[string]error)
 GATHER_RESULTS:
 	for {
@@ -787,21 +839,20 @@ GATHER_RESULTS:
 
 // tryDecrypt returns the plaintext and a flag indicating whether the decryption was done by the "unwrapSeal" (see
 // sealWrapMigration.Decrypt).
-func (a *access) tryDecrypt(ctx context.Context, sealWrapper *SealWrapper, ciphertextByKeyId map[string]*wrapping.BlobInfo, options []wrapping.Option) ([]byte, bool, error) {
+func (a *access) tryDecrypt(ctx context.Context, sealWrapper *SealWrapper, value *MultiWrapValue, options []wrapping.Option) ([]byte, bool, error) {
 	now := time.Now()
 	var decryptErr error
+	mLabels := []metrics.Label{{Name: "seal_wrapper_name", Value: sealWrapper.Name}}
+
 	defer func(now time.Time) {
-		metrics.MeasureSince([]string{"seal", "decrypt", "time"}, now)
-		metrics.MeasureSince([]string{"seal", sealWrapper.Name, "decrypt", "time"}, now)
+		metrics.MeasureSinceWithLabels([]string{"seal", "decrypt", "time"}, now, mLabels)
 
 		if decryptErr != nil {
-			metrics.IncrCounter([]string{"seal", "decrypt", "error"}, 1)
-			metrics.IncrCounter([]string{"seal", sealWrapper.Name, "decrypt", "error"}, 1)
+			metrics.IncrCounterWithLabels([]string{"seal", "decrypt", "error"}, 1, mLabels)
 		}
 	}(now)
 
-	metrics.IncrCounter([]string{"seal", "decrypt"}, 1)
-	metrics.IncrCounter([]string{"seal", sealWrapper.Name, "decrypt"}, 1)
+	metrics.IncrCounterWithLabels([]string{"seal", "decrypt"}, 1, mLabels)
 
 	var pt []byte
 
@@ -809,7 +860,7 @@ func (a *access) tryDecrypt(ctx context.Context, sealWrapper *SealWrapper, ciphe
 	var keyId string
 	if id, err := sealWrapper.Wrapper.KeyId(ctx); err == nil {
 		keyId = id
-		if ciphertext, ok := ciphertextByKeyId[keyId]; ok {
+		if ciphertext := value.BlobInfoForKeyId(keyId); ciphertext != nil {
 			pt, decryptErr = sealWrapper.Wrapper.Decrypt(ctx, ciphertext, options...)
 
 			sealWrapper.SetHealthy(decryptErr == nil || IsOldKeyError(decryptErr), now)
@@ -817,7 +868,7 @@ func (a *access) tryDecrypt(ctx context.Context, sealWrapper *SealWrapper, ciphe
 	}
 	// If we don't get a result, try all the slots
 	if pt == nil && decryptErr == nil {
-		for _, ciphertext := range ciphertextByKeyId {
+		for _, ciphertext := range value.Slots {
 			pt, decryptErr = sealWrapper.Wrapper.Decrypt(ctx, ciphertext, options...)
 			if decryptErr == nil {
 				// Note that we only update wrapper health for failures on exact key ID match,
@@ -902,22 +953,23 @@ func (a *access) GetShamirKeyBytes(ctx context.Context) ([]byte, error) {
 	return shamirWrapper.KeyBytes(ctx)
 }
 
-func slotsByKeyId(value *MultiWrapValue) map[string]*wrapping.BlobInfo {
-	ret := make(map[string]*wrapping.BlobInfo)
-	for _, blobInfo := range value.Slots {
-		keyId := ""
-		if blobInfo.KeyInfo != nil {
-			keyId = blobInfo.KeyInfo.KeyId
+func (v *MultiWrapValue) BlobInfoForKeyId(keyId string) *wrapping.BlobInfo {
+	for _, blobInfo := range v.Slots {
+		if blobInfo.KeyInfo != nil && blobInfo.KeyInfo.KeyId == keyId {
+			return blobInfo
 		}
-		ret[keyId] = blobInfo
 	}
-	return ret
+	return nil
 }
 
 type keyIdSet struct {
 	keyIds atomic.Pointer[[]string]
 }
 
+func (s *keyIdSet) initialized() bool {
+	return len(s.get()) > 0
+}
+
 func (s *keyIdSet) set(value *MultiWrapValue) {
 	keyIds := s.collect(value)
 	s.setIds(keyIds)
